/**
 * Created by JetBrains PhpStorm.
 * User: kasi
 * Date: 13.03.12
 * Time: 15:06
 * To change this template use File | Settings | File Templates.
 */

Tx.Component = Tx.extend(Tx.Base, {
    left:null,
    right:null,
    up:null,
    down:null,
    action:null,
    specialKey:null,
    initMouse:Tx.Device.hasMouse(),
    _events:null,
    constructor:function (options) {
        this.boundMouseOver = this.handleMouseOver.bind(this);
        this.boundMouseClick = this.handleMouseClick.bind(this);
        Tx.Base.prototype.constructor.apply(this, arguments);
        Tx.apply(this, options);
        if (this.element){
            var element = this.element;
            delete this.element;
            this.setElement(element);
        }
        this.initComponent();
    },
    setElement: function(element) {
        element = Tx.get(element);
        if (element){
            this.removeMouseEvents();
            this.element = element;
            element.initState();
            this.id = element.getId();
            if (this.initMouse){
                this.initMouseEvents();
            }
        }
    },
    getId: function(){
        return this.id;
    },
    getElement: function() {
        return this.element;
    },
    initComponent:function () {

    },
    initMouseEvents:function () {
        var el = this.element;
        if (el && el.dom){
            el.addEventListener('mouseover', this.boundMouseOver);
            el.addEventListener('click', this.boundMouseClick);
        }

    },
    removeMouseEvents: function() {
        var el = this.element;
        if (el && el.dom){
            el.removeEventListener('mouseover', this.boundMouseOver);
            el.removeEventListener('click', this.boundMouseClick);
        }
    },
    elementFn: function(fn){
        var el = this.element, args = Array.prototype.slice.call(arguments, 1);
        if(el){
            return el[fn].apply(el, args);
        }
        return null;
    },
    show: function(){this.elementFn('show'); this.visible = true; return this},
    hide: function(){this.elementFn('hide'); this.visible = false; return this},
    addClass: function(cls){this.elementFn('addClass', cls); return this},
    removeClass: function(cls){this.elementFn('removeClass', cls); return this},
    hasClass: function(cls){return this.elementFn('hasClass', cls)},
    setFocus: function(state){this.elementFn('setFocus', state); return this},
    setSelected: function(state){this.elementFn('setSelected', state); return this},
    setStyle: function(name, value){this.elementFn('setStyle', name, value); return this},
    attr: function(name, value){return this.elementFn('attr',name,value); },
    setHtml: function(html){return this.elementFn('setHtml', html);},

    handleMouseOver:function (event) {
        this.fireEvent('mouseover', [event]);
    },
    handleMouseClick:function (event) {
        this.fireEvent('click', [event]);
    },
    fadeIn: function() {
        Tx.fadeIn(this.element, {
            from:0,
            duration:500,
            onFinished:this.show.bind(this)
        })
    },
    fadeOut: function() {
        Tx.fadeOut(this.element, {
            from:1,
            duration:500,
            onFinished:this.hide.bind(this)
        })
    }
});

Tx.Control = Tx.extend(Tx.Component, {
    components:null,
    defaultItem: null,
    currentItem:null,
    greenAction:'green',
    redAction:'red',
    blueAction:'blue',
    yellowAction:'yellow',
    hasFocus: false,
    constructor:function (config) {
        this.components = {};
        this.boundOnComponentClick = this.onComponentClicked.bind(this);
        this.boundOnComponentOver = this.onComponentOver.bind(this);
        Tx.Component.prototype.constructor.apply(this, arguments);
    },
    add:function (component, cfg) {
        if (typeof component == 'function') {
            component = new component(cfg);
        }
        this.addComponent(component);
    },
    addComponent:function (component) {
        if (Tx.Device.hasMouse()) {
            component.addEvent('click', this.boundOnComponentClick);
            component.addEvent('mouseover', this.boundOnComponentOver);
        }
        this.components[component.getId()] = component;
        if (component.ref){
            this[component.ref] = component;
        }
    },
    onComponentClicked:function (component, event) {
        Tx.stopEvent(event);
        this.handleKey(TX_KEY_ENTER);
    },
    onComponentOver:function (component, event) {
        Tx.stopEvent(event);
        this.setFocus(component.getId());
    },
    show:function () {
        Tx.Component.prototype.show.apply(this, arguments);
        this.onShow();
    },
    hide:function () {
        Tx.Component.prototype.hide.apply(this, arguments);
        this.fireEvent('hide');
        this.onHide();
    },
    setData:function () {

    },
    setFocus:function (id) {
        if (id === true) {
            this.updateFocus();
            return;
        }
        if (id === false){
            this.hasFocus = false;
            return;
        }
        if (this.components[id] && (this.currentItem == null || this.currentItem != id)) {
            var el = this.getComponent();
            if (el) {
                el.setFocus(false);
                this.hasFocus = false;
            }
            this.currentItem = id;
            el = this.getComponent();
            if (el) {
                el.setFocus(true);
                this.hasFocus = true;
                this.onFocusChanged();
            }
        }
        else {
            this.updateFocus();
        }
    },
    updateFocus:function () {
        var el = this.getComponent();
        if (el) {
            el.setFocus(true);
            this.hasFocus = true;
        }
        else {
            if (this.defaultItem && this.components[this.defaultItem]){
                this.setFocus(this.defaultItem);
            }
            else {
                for (var key in this.components){
                    this.setFocus(this.components[key].getId());
                    return;
                }
            }
        }
    },
    getComponent:function (id) {
        return this.components[typeof id == 'undefined' ? this.currentItem : id];
    },
    onFocusChanged:function () {

    },
    handleKey:function (key) {
        var current = this.getComponent(), target, handled = false;
        if (current && current.handleKey) {
            target = current.handleKey(key, this);
            if (target === true) {
                return true;
            }
        }
        switch (key) {
            case TX_KEY_RED:
            case TX_KEY_YELLOW:
            case TX_KEY_BLUE:
            case TX_KEY_GREEN:
                return this.onSpecialKey(key);
        }
        if (!current) {
            return false;
        }
        if (!target) {
            switch (key) {
                case TX_KEY_LEFT:
                    target = current.left;
                    break;
                case TX_KEY_RIGHT:
                    target = current.right;
                    break;
                case TX_KEY_UP:
                    target = current.up;
                    break;
                case TX_KEY_DOWN:
                    target = current.down;
                    break;
                case TX_KEY_ENTER:
                    return this.doAction(current);
            }
        }
        if (typeof target == 'function') {
            target = target.call(current.scope || this, key, current, this);
        }
        if (target === true) {
            return true;
        }
        if (target === null || target === false || typeof target == 'undefined' || !this.components[target]) {
            switch (key) {
                case TX_KEY_LEFT:
                    return this.left;
                case TX_KEY_RIGHT:
                    return this.right;
                case TX_KEY_UP:
                    return this.up;
                case TX_KEY_DOWN:
                    return this.down;
                default:
                    return false;
            }
        }
        this.setFocus(target);
        return true;
    },
    doAction:function (el) {
        el = el || this.getComponent();
        if (!el) return;
        if (this.onAction(el) === true) {
            return true;
        }
        var action = el.action, data = el.data;
        if (action) {
            this.fireEvent('action', [action, data, el]);
            el.setSelected(true);
            return true;
        }
        if (el.specialKey) {
            return this.onSpecialKey(el.specialKey);
        }
    },
    onShow:function () {
        this.setFocus(this.currentItem || this.startItem);
    },
    onHide:function () {

    },
    onAction:function () {
    },
    onSpecialKey:function (key) {
        var action = false;
        switch (key) {
            case TX_KEY_GREEN:
                action = this.greenAction;
                break;
            case TX_KEY_RED:
                action = this.redAction;
                break;
            case TX_KEY_BLUE:
                action = this.blueAction;
                break;
            case TX_KEY_YELLOW:
                action = this.yellowAction;
                break;
        }
        if (action) {
            this.fireEvent('action', [action]);
            return true;
        }
        return false;
    }
});

Tx.Page = Tx.extend(Tx.Control, {
});
Tx.Container = Tx.extend(Tx.Component, {
    items:null,
    selectedIndex:null,
    selectable:true,
    multiselect:false,
    focusIndex:-1,
    defaultFocus:0,
    vertical:false,
    empty:false,
    contentEl:null,
    initComponent:function () {
        Tx.Component.prototype.initComponent.apply(this, arguments);
        this.items = [];
        this.contentEl = Tx.get(this.contentEl || this.element);
    },
    getContentElement:function () {
        return this.contentEl;
    },
    clearContent: function() {
        var el = this.getContentElement();
        this.removeItems();
        if (el){
            el.setHtml('');
        }
    },
    removeItems:function () {
        this.getContentElement().setHtml('');
        for (var i = 0, items = this.items, l = items.length, item; i < l; i++) {
            item = items[i];
            if (item.removeAllEvents()){
                item.removeAllEvents();
            }
            item.removeEvent(this.boundMouseClick);
            item.removeEvent(this.boundMouseOver);
        }
        this.items = [];
    },
    initMouseEvents:Tx.empty,
    getValue:function () {
        if (this.selectedIndex >= 0) {
            var item = this.getItem(this.selectedIndex);
            if (item) {
                return item.value || this.value || '';
            }
        }
        return '';
    },
    setFocus:function (state) {
        Tx.Component.prototype.setFocus.call(this, state);
        if (state) {
            this.hasFocus = true;
            this.focusItem(this.focusIndex == -1 ? this.defaultFocus : this.focusIndex);
        }
        else {
            this.clearFocus();
            this.hasFocus = false;
        }
    },
    clearFocus:function () {
        this.hasFocus = false;
        var item = this.getItem(this.focusIndex);
        if (item) {
            item.setFocus(false);
        }
    },
    focusItem:function (id) {
        this.clearFocus();
        var item = this.getItem(id);
        if (item) {
            item.setFocus(true);
            this.hasFocus = true;
            this.focusIndex = id;
        }
        else {
            this.focusIndex = -1;
        }
    },
    clearSelection:function (noChange) {
        for (var i = 0, l = this.items.length; i < l; i++) {
            this.items[i].setSelected(false);
        }
        if (noChange === true){
            this.selectedIndex = null;
        }
    },
    selectItem:function (id) {
        var item;
        if (!this.selectable) {
            return;
        }
        if (!this.multiselect) {
            this.clearSelection(true);
        }
        item = this.getItem(id);
        if (item) {
            item.setSelected(true);
            if (this.selectedIndex != id) {
                this.fireEvent('change', [item]);
            }
            this.selectedIndex = id;
        }

    },
    add:function (component, cfg) {
        if (typeof component == 'function') {
            component = new component(cfg);
        }
        this.addItem(component);
    },
    addItem:function (item) {
        if (!item) {
            return;
        }
        this.items.push(item);
        var i = this.items.length - 1;
        item.listIndex = i;
        if (item.dom) {
            item.dom.txListIndex = i;
        }
        if (this.initMouse) {
            item.addEvent('mouseover', this.boundMouseOver);
            item.addEvent('click', this.boundMouseClick);
        }
    },
    setData: function(data){
        this.data = data;
        this.setItems();
    },
    setItems: function() {
        for(var i= 0, l = this.items.length; i<l; i++){
            this.setItem(this.items[i], this.data[i])
        }
    },
    setItem: function(item, data){
        item.setItem(data.name);
    },
    updateListIndex:function () {
        var items = this.items, item;
        for (var i = 0, l = items.length; i < l; i++) {
            item = items[i];
            if (item && item.dom) {
                item.listIndex = i;
                item.dom.txListIndex = i;
            }
        }
    },
    getItem:function (index) {
        return this.items[index];
    },
    getFocusItem: function() {
        return this.items[this.focusIndex];
    },
    focusNext:function () {
        if (this.focusIndex >= this.items.length - 1) {
            return this.vertical ? this.down : this.right;
        }
        this.focusItem(this.focusIndex + 1);
        return true;
    },
    focusPrev:function () {
        if (this.focusIndex > 0) {
            this.focusItem(this.focusIndex - 1);
            return true;
        }
        else {
            return this.vertical ? this.up : this.left;
        }
    },
    focusLast:function () {
        if (this.items.length > 0) {
            this.focusItem(this.items.length - 1);
            return null;
        }
        else {
            return this.empty || false;
        }
    },
    handleKey:function (key) {
        if (this.vertical) {
            switch (key) {
                case TX_KEY_DOWN:
                    return this.focusNext();
                case TX_KEY_UP:
                    return this.focusPrev();
            }
        }
        else {
            switch (key) {
                case TX_KEY_LEFT:
                    return this.focusPrev();
                case TX_KEY_RIGHT:
                    return this.focusNext();
            }
        }
        switch (key) {
            case TX_KEY_LEFT:
                return this.left;
            case TX_KEY_RIGHT:
                return this.right;
            case TX_KEY_UP:
                return this.up;
            case TX_KEY_DOWN:
                return this.down;
            case TX_KEY_ENTER:
                if (this.focusIndex >= 0) {
                    this.selectItem(this.focusIndex);
                    this.onAction();
                }
                break;
        }
    },
    onAction:function () {
        var item = this.getItem(this.selectedIndex);
        if (item) {
            this.fireEvent('action', [item]);
        }
    },
    handleMouseOver:function (item) {
        this.focusItem(item.listIndex);
    },
    handleMouseClick:function (event) {
        this.handleKey(TX_KEY_ENTER);
    }
});
Tx.ScrollContainer = Tx.extend(Tx.Container, {
    vertical:false,
    dataOffset: 0,
    prevArrow: null,
    nextArrow: null,
    initComponent: function() {
        Tx.Container.prototype.initComponent.apply(this, arguments);
        if (this.prevArrow){
            this.prevArrow = Tx.get(this.prevArrow);
            if (this.prevArrow) {
                this.prevArrow.addEventListener('mouseover', this.onPrevArrowOver.bind(this));
                this.prevArrow.addEventListener('click', this.onPrevArrowClick.bind(this));

            }
        }
        if (this.nextArrow){
            this.nextArrow = Tx.get(this.nextArrow);
            if (this.nextArrow) {
                this.nextArrow.addEventListener('mouseover', this.onNextArrowOver.bind(this));
                this.nextArrow.addEventListener('click', this.onNextArrowClick.bind(this));

            }
        }
        this.data = [];
    },
    onPrevArrowOver: function(event){
        this.fireEvent('prevover');
    },
    onNextArrowOver: function(event){
        this.fireEvent('nextover');
    },
    focusItem: function(){
        Tx.Container.prototype.focusItem.apply(this, arguments);
        this.updateArrows();
    },
    onPrevArrowClick: function() {
        this.handleKey(this.vertical ? TX_KEY_UP : TX_KEY_LEFT);
    },
    onNextArrowClick: function() {
        this.handleKey(this.vertical ? TX_KEY_DOWN : TX_KEY_RIGHT);
    },
    scrollPrev: function() {
        if (this.canScrollPrev()){
            this.dataOffset--;
            this.setItems();
            this.updateArrows();
            return true;
        }
        return this.vertical ? this.up : this.left;
    },
    scrollNext: function() {
        if (this.canScrollNext()){
            this.dataOffset++;
            this.setItems();
            this.updateArrows();
            return true;
        }
        return this.vertical ? this.down : this.right;
    },
    focusLast: function() {

    },
    canScrollPrev: function() {
        return this.dataOffset > 0;
    },
    canScrollNext: function() {
        return this.dataOffset + this.items.length < this.data.length;
    },
    focusNext: function() {
        var result;
        if (this.focusIndex >= this.data.length-1){
            result = this.vertical ? this.down : this.right;
        }
        else if (this.focusIndex >= this.items.length -1){
            result = this.scrollNext();
        }
        else {
            result = Tx.Container.prototype.focusNext.apply(this, arguments);
        }
        this.updateArrows();
        return result;
    },
    focusPrev: function() {
        var result;
        if (this.focusIndex == 0){
            result = this.scrollPrev();
        }
        else {
            result = Tx.Container.prototype.focusPrev.apply(this, arguments);
        }
        this.updateArrows();
        return result;
    },
    setData: function(data){
        this.data = data;
        this.dataOffset = 0;
        this.setItems();
        this.updateArrows();
    },
    setItems: function() {
        for(var i= 0, l = this.items.length; i<l; i++){
            this.setItem(this.items[i], this.data[i+this.dataOffset])
        }
    },
    setItem: function(item, data){
        item.setItem(data.name);
    },
    updateArrows: function(){
        if (this.prevArrow){
            if (this.canScrollPrev()){
                this.prevArrow.show();
            }
            else {
                this.prevArrow.hide();
            }
        }
        if (this.nextArrow){
            if (this.canScrollNext()){
                this.nextArrow.show();
            }
            else {
                this.nextArrow.hide();
            }
        }
    }
});

Tx.Textfield = Tx.extend(Tx.Component, {
    keyboardVisible:false,
    keyboardMode: 'default',
    emptyValue:'',
    txType: 'textfield',
    showOnFocus: true,
    constructor: function(id, options){
        Tx.Component.prototype.constructor.apply(this, arguments);
        this.input = Tx.get(this.input);
        /*
        if (!Tx.Device.hasKeyboard() && this.input.dom && this.input.dom.tagName.toLowerCase() == 'input'){
            var div = document.createElement('div'), cls = this.input.dom.className, id=this.input.getId();
            div.className = cls;
            div.setAttribute('id', id);
            this.input.destroy();
            this.input.dom = div;
            this.element.insertChild(div);
            this.input.addClass('input');
            this.isDivInput = true;
        }
        */
        this.setValue(this.value || this.emptyValue);
    },
    getValue: function(){
        if (this.isDivInput){
            return this.input.getHtml();
        }
        else {
            return this.input.dom.value || '';
        }
    },
    setValue: function(value){
        if (this.limit && String(value).length > this.limit){
            return;
        }
        if (value == ''){
            value = this.emptyValue;
        }
        this.value = value;
        this.setFieldValue(value);
        if (value == this.emptyValue || value == ''){
            this.input.addClass('empty');
        }
        else {
            this.input.removeClass('empty');
        }
    },
    setFieldValue: function(value) {
        if (this.isDivInput){
            this.input.setHtml(value);
        }
        else {
            this.input.dom.value = value;
        }
    },
    setFocus: function(state){
        Tx.Component.prototype.setFocus.apply(this, arguments);
        if (!!state){
            //this.showKeyboard();
            if (Tx.Device.hasKeyboard()){
                this.input.dom.focus();
            }
            else {
                if (this.showOnFocus){
                    this.showKeyboard();
                }
            }
        }
        else {
            this.hideKeyboard();
            if (Tx.Device.hasKeyboard()){
                this.input.dom.blur();
                //document.getElementById('fake-anchor').focus();
            }
        }
    },
    addString: function(str) {
        var value = this.getValue();
        if (value == this.emptyValue){
            value = str;
        }
        else {
            value = value + str;
        }
        this.setValue(value);
    },
    backspace: function() {
        var value = this.getValue();
        if (value.length > 0){
            value = value.substr(0,value.length-1);
            this.setValue(value);
        }
    },
    clear: function() {
        this.setValue(this.emptyValue);
    },
    handleKey: function(key){
        switch(key){
            default:
                if (this.keyboardVisible){
                    return Tx.SimpleKeyboard.handleKey(key);
                }
                else {
                    if (this.getValue() == this.emptyValue){
                        this.setFieldValue('');
                    }
                }
        }
        return false;
    },
    handleMouseClick: function(event) {
        Tx.log('Tx.TextField::handleMouseClick, do not fire event');
//        this.fireEvent('click', [event]);
    },
    hideKeyboard: function() {
        Tx.SimpleKeyboard.hide();
        this.keyboardVisible = false;
    },
    showKeyboard: function() {
        if (!Tx.Device.hasKeyboard()){
            Tx.SimpleKeyboard.show(this);
            this.keyboardVisible = true;
        }
    },
    kbdInsert: function(str){
        this.addString(str);
    },
    kbdBackspace: function(){
        this.backspace();
    },
    kbdClear: function() {
        this.clear();
    },
    kbdEnter: function() {
        this.hideKeyboard();
        this.fireEvent('enter');
    },
    kbdExit: function() {
        this.hideKeyboard();
        this.fireEvent('exit');
    }
});

Tx.PasswordField = Tx.extend(Tx.Textfield, {
    getValue: function(){
        var value = (this.isDivInput ? this.value : this.input.dom.value) || '';
        if (value == this.emptyValue){
            value = '';
        }
        return value;
    },
    setValue: function(value){
        var valLength = String(value).length;
        if (value == ''){
            value = this.emptyValue;
        }
        if (Tx.Device.hasKeyboard() && value == this.emptyValue){
            value = '';
        }
        this.value = value;
        if (this.isDivInput){
            this.input.setHtml(value == this.emptyValue ? this.emptyValue :'*************************************'.substr(0,valLength));
        }
        else {
            this.input.dom.value = value;
        }
        if (value == this.emptyValue || value == ''){
            this.input.addClass('empty');
        }
        else {
            this.input.removeClass('empty');
        }
    },
    onShow: function() {
        this.value = '';
        Tx.Textfield.prototype.onShow.apply(this, arguments);
    }
});
Tx.Keyboard = new (Tx.extend(Tx.Base, {
    row:0,
    col:0,
    top:true,
    focusEl:null,
    initialized:false,
    visible:false,
    init:function () {
        if (this.initialized) {
            return;
        }
        if (!Tx.byId('keyboard')) {
            var div = document.createElement('div');
            Tx.fly(div).attr('id', 'keyboard');
            Tx.getBody().appendChild(div);
        }
        this.element = Tx.get('keyboard');
        this.element.hide();
        this.visible = false;
        this.element.addEventListener('mouseover', this.handleMouseOver.bind(this));
        this.element.addEventListener('click', this.handleMouseClick.bind(this));
    },
    handleMouseOver:function (event) {
        var el = Tx.fly(event.originalTarget || event.toElement || event.target), id = el.attr('id'), chr, code, codeStr, key;
        if (el.dom.tagName.toLowerCase() == 'li' && id) {
            codeStr = id.substr(6);
            if (codeStr) {
                key = this.keys[String.fromCharCode(parseInt(codeStr))];
                if (!key) {
                    key = this.keys[codeStr];
                }
                if (key && key.left && key.right && key.down && key.up) {
                    this.setFocus(key.id);
                }
            }
        }
    },
    handleMouseClick:function (event) {
        this.handleKey(TX_KEY_ENTER);
    },
    show:function (input, mode) {
        this.input = input;
        switch(mode){
            case 'compact': this.renderCompact(); break;
            case 'numeric': this.renderNumeric(); break;
            default: this.renderDefault(); break;
        }
        this.element.show();
        this.visible = true;
    },
    hide:function () {
        if(this.element && this.element.dom){
            this.element.hide();
        }
        this.visible = false;
    },
    defineKeys:function () {
        function add(id, cls, chr, chr2, chr3, left, right, up, down, action) {
            return {
                id:id,
                cls:cls,
                chr:chr,
                chr2:chr2,
                chr3:chr3,
                left:left,
                right:right,
                up:up,
                down:down,
                action:action
            }
        }

        /*
         blue 1 2 3 4 5 6 7 8 9 0 @ . / back
         yell a b c d e f g h i j k l m clear
         gree n o p q r s t u v w x y z enter
         - - - - space     - - - - exit


         */
        return [
            [
                add('blue', 'symbol blue', 'ABC', '', '', 'back', '1', 'green', 'yellow', 'blue'),
                add('1', 'letter', '1', '!', '', 'blue', '2', 'de', 'a', 'ins'),
                add('2', 'letter', '2', '"', '', '1', '3', 'com', 'b', 'ins'),
                add('3', 'letter', '3', '§', '', '2', '4', 'tv', 'c', 'ins'),
                add('4', 'letter', '4', '$', '', '3', '5', 'net', 'd', 'ins'),
                add('5', 'letter', '5', '%', '', '4', '6', 'space', 'e', 'ins'),
                add('6', 'letter', '6', '&amp;', '', '5', '7', 'space', 'f', 'ins'),
                add('7', 'letter', '7', '/', '', '6', '8', 'space', 'g', 'ins'),
                add('8', 'letter', '8', '(', '', '7', '9', 'space', 'h', 'ins'),
                add('9', 'letter', '9', ')', '', '8', '0', 'space', 'i', 'ins'),
                add('0', 'letter', '0', '=', '', '9', '@', 'space', 'j', 'ins'),
                add('@', 'letter', '@', '?', '', '0', '.', 'space', 'k', 'ins'),
                add('.', 'letter', '.', '+', '', '@', '/', 'space', 'l', 'ins'),
                add('/', 'letter', '/', '-', '', '.', 'back', 'space', 'm', 'ins'),
                add('back', 'symbol bsp', '', '', '', '/', 'blue', 'exit', 'clear', 'backspace')
            ],
            [
                add('yellow', 'symbol yellow', 'ALT', '', '', 'clear', 'a', 'blue', 'green', 'yellow'),
                add('a', 'letter', 'a', 'A', 'ä', 'yellow', 'b', '1', 'n', 'ins'),
                add('b', 'letter', 'b', 'B', 'ö', 'a', 'c', '2', 'o', 'ins'),
                add('c', 'letter', 'c', 'C', 'ü', 'b', 'd', '3', 'p', 'ins'),
                add('d', 'letter', 'd', 'D', 'Ä', 'c', 'e', '4', 'q', 'ins'),
                add('e', 'letter', 'e', 'E', 'Ö', 'd', 'f', '5', 'r', 'ins'),
                add('f', 'letter', 'f', 'F', 'Ü', 'e', 'g', '6', 's', 'ins'),
                add('g', 'letter', 'g', 'G', 'ß', 'f', 'h', '7', 't', 'ins'),
                add('h', 'letter', 'h', 'H', 'é', 'g', 'i', '8', 'u', 'ins'),
                add('i', 'letter', 'i', 'I', 'É', 'h', 'j', '9', 'v', 'ins'),
                add('j', 'letter', 'j', 'J', 'ó', 'i', 'k', '0', 'w', 'ins'),
                add('k', 'letter', 'k', 'K', 'Ó', 'j', 'l', '@', 'x', 'ins'),
                add('l', 'letter', 'l', 'L', 'ú', 'k', 'm', '.', 'y', 'ins'),
                add('m', 'letter', 'm', 'M', 'Ú', 'l', 'clear', '/', 'z', 'ins'),
                add('clear', 'symbol clear', 'clear', '', '', 'm', 'yellow', 'back', 'enter', 'clear')
            ],
            [
                add('green', 'symbol', 'http://', '', '', 'enter', 'n', 'yellow', 'blue', 'ins'),
                add('n', 'letter', 'n', 'N', '#', 'green', 'o', 'a', 'de', 'ins'),
                add('o', 'letter', 'o', 'O', '+', 'n', 'p', 'b', 'com', 'ins'),
                add('p', 'letter', 'p', 'P', '*', 'o', 'q', 'c', 'tv', 'ins'),
                add('q', 'letter', 'q', 'Q', ';', 'p', 'r', 'd', 'net', 'ins'),
                add('r', 'letter', 'r', 'R', ':', 'q', 's', 'e', 'space', 'ins'),
                add('s', 'letter', 's', 'S', '_', 'r', 't', 'f', 'space', 'ins'),
                add('t', 'letter', 't', 'T', '>', 's', 'u', 'g', 'space', 'ins'),
                add('u', 'letter', 'u', 'U', '<', 't', 'v', 'h', 'space', 'ins'),
                add('v', 'letter', 'v', 'V', '|', 'u', 'w', 'i', 'space', 'ins'),
                add('w', 'letter', 'w', 'W', '€', 'v', 'x', 'j', 'space', 'ins'),
                add('x', 'letter', 'x', 'X', '', 'w', 'y', 'k', 'space', 'ins'),
                add('y', 'letter', 'y', 'Y', '', 'x', 'z', 'l', 'space', 'ins'),
                add('z', 'letter', 'z', 'Z', '', 'y', 'enter', 'm', 'space', 'ins'),
                add('enter', 'symbol enter', '', '', '', 'z', 'green', 'clear', 'exit', 'enter')
            ],
            [
                add('k1', 'symbol', '', '', '', '', '', '', '', ''),
                add('de', 'letter', '.de', '', '', 'exit', 'com', 'n', '1', 'ins'),
                add('com', 'letter', '.com', '', '', 'de', 'tv', 'o', '2', 'ins'),
                add('tv', 'letter', '.tv', '', '', 'com', 'net', 'p', '3', 'ins'),
                add('net', 'letter', '.net', '', '', 'tv', 'space', 'q', '4', 'ins'),
                add('space', 'space', '', '', '', 'net', 'exit', 't', '7', 'space'),
                add('k7', 'letter', '', '', '', '', '', '', '', ''),
                add('k8', 'letter', '', '', '', '', '', '', '', ''),
                add('k9', 'letter', '', '', '', '', '', '', '', ''),
                add('k10', 'letter', '', '', '', '', '', '', '', ''),
                add('exit', 'symbol exit', '', '', '', 'space', 'de', 'enter', 'back', 'exit')
            ]
        ];
    },
    defineKeysCompact:function () {
        function add(id, cls, chr, chr2, chr3, left, right, up, down, action) {
            return {
                id:id,
                cls:cls,
                chr:chr,
                chr2:chr2,
                chr3:chr3,
                left:left,
                right:right,
                up:up,
                down:down,
                action:action
            }
        }

        /*
         blue 1 2 3 4 5 6 7 8 9 0 back
         yell a b c d e f g h i j clear
         gree k l m n o p q r s t enter
         nix  u v w x y z @ . / - exit
         nix  - - - space - - -   nix


         */
        return [
            [
                add('blue', 'symbol blue', 'ABC', '', '', 'back', '1', 'green', 'yellow', 'blue'),
                add('1', 'letter', '1', '!', '', 'blue', '2', 'space', 'a', 'ins'),
                add('2', 'letter', '2', '"', '', '1', '3', 'space', 'b', 'ins'),
                add('3', 'letter', '3', '§', '', '2', '4', 'space', 'c', 'ins'),
                add('4', 'letter', '4', '$', '', '3', '5', 'space', 'd', 'ins'),
                add('5', 'letter', '5', '%', '', '4', '6', 'space', 'e', 'ins'),
                add('6', 'letter', '6', '&', '', '5', '7', 'space', 'f', 'ins'),
                add('7', 'letter', '7', '/', '', '6', '8', 'space', 'g', 'ins'),
                add('8', 'letter', '8', '(', '', '7', '9', 'space', 'h', 'ins'),
                add('9', 'letter', '9', ')', '', '8', '0', 'space', 'i', 'ins'),
                add('0', 'letter', '0', '=', '', '9', 'back', 'space', 'j', 'ins'),
                add('back', 'symbol bsp', '', '', '', '0', 'blue', 'exit', 'clear', 'backspace')
            ],
            [
                add('yellow', 'symbol yellow', 'ALT', '', '', 'clear', 'a', 'blue', 'green', 'yellow'),
                add('a', 'letter', 'a', 'A', 'ä', 'yellow', 'b', '1', 'k', 'ins'),
                add('b', 'letter', 'b', 'B', 'ö', 'a', 'c', '2', 'l', 'ins'),
                add('c', 'letter', 'c', 'C', 'ü', 'b', 'd', '3', 'm', 'ins'),
                add('d', 'letter', 'd', 'D', 'Ä', 'c', 'e', '4', 'n', 'ins'),
                add('e', 'letter', 'e', 'E', 'Ö', 'd', 'f', '5', 'o', 'ins'),
                add('f', 'letter', 'f', 'F', 'Ü', 'e', 'g', '6', 'p', 'ins'),
                add('g', 'letter', 'g', 'G', 'ß', 'f', 'h', '7', 'q', 'ins'),
                add('h', 'letter', 'h', 'H', 'é', 'g', 'i', '8', 'r', 'ins'),
                add('i', 'letter', 'i', 'I', 'É', 'h', 'j', '9', 's', 'ins'),
                add('j', 'letter', 'j', 'J', 'ó', 'i', 'clear', '0', 't', 'ins'),
                add('clear', 'symbol clear', 'clear', '', '', 'j', 'yellow', 'back', 'enter', 'clear')
            ],
            [
                add('green', 'symbol green', '', '', '', 'enter', 'k', 'yellow', 'blue', 'green'),
                add('k', 'letter', 'k', 'K', 'Ó', 'green', 'l', 'a', 'u', 'ins'),
                add('l', 'letter', 'l', 'L', 'ú', 'k', 'm', 'b', 'v', 'ins'),
                add('m', 'letter', 'm', 'M', 'Ú', 'l', 'n', 'c', 'w', 'ins'),
                add('n', 'letter', 'n', 'N', '#', 'm', 'o', 'd', 'x', 'ins'),
                add('o', 'letter', 'o', 'O', '+', 'n', 'p', 'e', 'y', 'ins'),
                add('p', 'letter', 'p', 'P', '*', 'o', 'q', 'f', 'z', 'ins'),
                add('q', 'letter', 'q', 'Q', ';', 'p', 'r', 'g', '@', 'ins'),
                add('r', 'letter', 'r', 'R', ':', 'q', 's', 'h', '.', 'ins'),
                add('s', 'letter', 's', 'S', '_', 'r', 't', 'i', '/', 'ins'),
                add('t', 'letter', 't', 'T', '>', 's', 'enter', 'j', '-', 'ins'),
                add('enter', 'symbol enter', '', '', '', 't', 'green', 'clear', 'exit', 'enter')
            ],
            [
                add('0-1', 'symbol', '', '', '', '', '', '', '', ''),
                add('u', 'letter', 'u', 'U', '<', 'exit', 'v', 'k', 'space', 'ins'),
                add('v', 'letter', 'v', 'V', '|', 'u', 'w', 'l', 'space', 'ins'),
                add('w', 'letter', 'w', 'W', '€', 'v', 'x', 'm', 'space', 'ins'),
                add('x', 'letter', 'x', 'X', '', 'w', 'y', 'n', 'space', 'ins'),
                add('y', 'letter', 'y', 'Y', '', 'x', 'z', 'o', 'space', 'ins'),
                add('z', 'letter', 'z', 'Z', '', 'y', '@', 'p', 'space', 'ins'),
                add('@', 'letter', '@', '?', '', 'z', '.', 'q', 'space', 'ins'),
                add('.', 'letter', '.', '+', '', '@', '/', 'r', 'space', 'ins'),
                add('/', 'letter', '/', '/', '', '.', '-', 's', 'space', 'ins'),
                add('-', 'letter', '-', '-', '', '/', 'exit', 't', 'space', 'ins'),
                add('exit', 'symbol exit', '', '', '', '-', 'u', 'enter', 'back', 'exit')
            ],
            [
                add('k1', 'symbol', '', '', '', '', '', '', '', ''),
                add('k3', 'letter', '', '', '', '', '', '', '', ''),
                add('k4', 'letter', '', '', '', '', '', '', '', ''),
                add('k5', 'letter', '', '', '', '', '', '', '', ''),
                add('space', 'space', ' ', '', '', '', '', 'z', '6', 'space'),
                add('k9', 'letter', '', '', '', '', '', '', '', ''),
                add('k10', 'letter', '', '', '', '', '', '', '', ''),
                add('k11', 'symbol', '', '', '', '', '', '', '', '')
            ]
        ];
    },
    defineKeysNumeric:function () {
        return [
            [
                {id:"1", cls:'letter', chr:'1', left:'back', right:'2', up:'.', down:'4', action:'ins'},
                {id:"2", cls:'letter', chr:'2', left:'1', right:'3', up:'0', down:'5', action:'ins'},
                {id:"3", cls:'letter', chr:'3', left:'2', right:'back', up:',', down:'6', action:'ins'},
                {id:"back", cls:'symbol bsp', chr:'', left:'3', right:'1', up:'exit', down:'clear', action:'backspace'}
            ],
            [
                {id:"4", cls:'letter', chr:'4', left:'clear', right:'5', up:'1', down:'7', action:'ins'},
                {id:"5", cls:'letter', chr:'5', left:'4', right:'6', up:'2', down:'8', action:'ins'},
                {id:"6", cls:'letter', chr:'6', left:'5', right:'clear', up:'3', down:'9', action:'ins'},
                {id:"clear", cls:'symbol clear', chr:'clear', left:'6', right:'4', up:'back', down:'enter', action:'clear'}
            ],
            [
                {id:"7", cls:'letter', chr:'7', left:'enter', right:'8', up:'4', down:'.', action:'ins'},
                {id:"8", cls:'letter', chr:'8', left:'7', right:'9', up:'5', down:'0', action:'ins'},
                {id:"9", cls:'letter', chr:'9', left:'8', right:'enter', up:'6', down:',', action:'ins'},
                {id:"enter", cls:'symbol enter', chr:'', left:'9', right:'7', up:'clear', down:'exit', action:'enter'}
            ],
            [
                {id:".", cls:'letter', chr:'.', left:'exit', right:'0', up:'7', down:'1', action:'ins'},
                {id:"0", cls:'letter', chr:'0', left:'.', right:',', up:'8', down:'2', action:'ins'},
                {id:",", cls:'letter', chr:',', left:'0', right:'exit', up:'9', down:'3', action:'ins'},
                {id:"exit", cls:'symbol exit', chr:'', left:',', right:'.', up:'enter', down:'back', action:'exit'}
            ]
        ]
    },
    setFocus:function (id) {
        var el, key = this.keys[id];
        if (!key) {
            return;
        }
        if (this.focusEl) {
            Tx.fly(this.focusEl).removeClass('focus');
        }
        el = Tx.byId(this.getKeyId(id));
        if (el) {
            Tx.fly(el).addClass('focus');
            this.focusEl = el;
            this.currentKey = key;
        }
    },
    getKeyId:function (id) {
        id = String(id);
        if (id.length == 1) {
            id = id.charCodeAt(0);
        }
        return 'txkey-' + id;
    },
    render:function (keys) {
        if (this.mode) {
            this.element.removeClass(this.mode);
        }
        var i, x, y, ylen, xlen, row, html = [], key, defs = {};
        for (y = 0, ylen = keys.length; y < ylen; y++) {
            html.push('<ul>');
            row = keys[y];
            for (x = 0, xlen = row.length; x < xlen; x++) {
                key = row[x];
                defs[key.id] = key;
                html.push('<li id="' + this.getKeyId(key.id) + '" class="' + key.cls + '">' + Tx.htmlspecialchars(key.chr) + '</li>')
            }
            html.push('</ul>');
        }
        this.keys = defs;
        this.currentSet = 'chr';
        this.element.setHtml(html.join(''));
    },
    renderDefault:function () {
        if (this.mode == 'default'){
            return;
        }
        this.render(this.defineKeys())
        this.setFocus('a');
        this.mode = 'default';
        this.element.addClass('default');
    },
    renderCompact:function () {
        if (this.mode == 'compact'){
            return;
        }
        this.render(this.defineKeysCompact())
        this.setFocus('e');
        this.mode = 'compact';
        this.element.addClass('compact');
    },
    renderNumeric:function () {
        if (this.mode == 'numeric'){
            return;
        }
        this.render(this.defineKeysNumeric());
        this.setFocus('5');
        this.mode = 'numeric';
        this.element.addClass('numeric');
    },
    handleKey:function (key) {
        var target, currentKey = this.onKeyPress(this.currentKey);
        if (!key || !currentKey) {
            return false;
        }
        switch (key) {
            case TX_KEY_ENTER:
                this.doAction(currentKey.action, currentKey);
                break;
            case TX_KEY_YELLOW:
                this.doAction('yellow');
                break;
            case TX_KEY_GREEN:
                this.doAction('green');
                break;
            case TX_KEY_BLUE:
                this.doAction('blue');
                break;
            default:
                target = this.getTarget(currentKey, key);
                if (target) {
                    this.setFocus(target);
                    return true;
                }
                break;
        }
        return true;
    },
    getTarget:function (currentKey, key) {
        switch (key) {
            case TX_KEY_LEFT:
                return currentKey.left;
                break;
            case TX_KEY_RIGHT:
                return currentKey.right;
                break;
            case TX_KEY_UP:
                return currentKey.up;
                break;
            case TX_KEY_DOWN:
                return currentKey.down;
                break;
            default:
                return null;
        }

    },
    setLetters:function (name) {
        var k, key, keys = this.keys;
        this.currentSet = name;
        for (k in keys) {
            if (keys.hasOwnProperty(k)) {
                key = keys[k];
                if (!key[name]) {
                    continue;
                }
                el = Tx.fly(this.getKeyId(key.id));
                if (el.dom) {
                    el.setHtml(Tx.htmlspecialchars(key[name]));
                }
            }
        }
    },
    doAction:function (action, currentKey) {
        this.fireEvent('keypressed', action, currentKey);
        switch (action) {
            case 'blue':
                this.setLetters(this.currentSet == 'chr' ? 'chr2' : 'chr');
                break;
            case 'yellow':
                this.setLetters(this.currentSet == 'chr' ? 'chr3' : 'chr');
                break;
            case 'green':
                if (this.toggleOnGreen){
                    this.top = !this.top;
                    if (this.top){
                        this.element.setStyles({top:'25px',bottom:'auto'});
                    }
                    else {
                        this.element.setStyles({top:'auto',bottom:'40px'});
                    }
                }
                break;
            case 'backspace':
                this.doBackspace();
                break;
            case 'clear':
                this.doClear();
                break;
            case 'enter':
                this.doEnter();
                break;
            case 'exit':
                this.doExit();
                break;
            case 'space':
                this.doInsert(' ');
                break;
            case 'ins':
                if (currentKey && (currentKey[this.currentSet] || currentKey.chr)){
                    this.doInsert(currentKey[this.currentSet] || currentKey.chr);
                }
                break;
        }
    },
    doInsert: function(str){
        if (this.input){
            this.input.kbdInsert(str);
        }
        this.fireEvent('insert', str);
    },
    onKeyPress:function (key) {
        return key;
    },
    doBackspace:function () {
        if (this.input){
            this.input.kbdBackspace();
        }
        this.fireEvent('backspace');
    },
    doClear:function () {
        if (this.input){
            this.input.kbdClear();
        }
        this.fireEvent('clear')
    },
    doEnter:function () {
        if (this.input){
            this.input.kbdEnter();
        }
        this.fireEvent('enter');
    },
    doExit:function () {
        this.hide();
        if (this.input){
            this.input.kbdExit();
        }
        this.fireEvent('exit');

    }
}));

Tx.SimpleKeyboard = new (Tx.extend(Tx.Base, {
    visible: false,
    initialized: false,
    currentText: 0,
    row:2,
    col:5,
    focusEl:null,
    lastNumberKey: false,
    numberKeyIndex: 0,
    init: function() {
        if (this.initialized) {
            return;
        }
        if (!Tx.byId('keyboard')) {
            var div = document.createElement('div');
            Tx.fly(div).attr('id', 'keyboard');
            Tx.getBody().appendChild(div);
        }
        this.element = Tx.get('keyboard');
        this.element.hide();
        this.visible = false;
        this.element.addEventListener('mouseover', this.handleMouseOver.bind(this));
        this.element.addEventListener('click', this.handleMouseClick.bind(this));
        this.text0 = [
            ['1','2','3','4','5','6','7','8','9','0','Ende'],
            ['a','b','c','d','e','f','g','h','i','j','ABC'],
            ['k','l','m','n','o','p','q','r','s','t','Eingabe'],
            ['u','v','w','x','y','z','ä','ö','ü','ß','Zurück'],
            ['.','/','@','+','-','&',',','*','#','?','Löschen'],
            ['.de','.com','Leer','.net','.tv']
        ];
        this.text1 = [
            ['!','"','§','$','%','&','/','(',')','=','Ende'],
            ['A','B','C','D','E','F','G','H','I','J','abc'],
            ['K','L','M','N','O','P','Q','R','S','T','Eingabe'],
            ['U','V','W','X','Y','Z','Ä','Ö','Ü','ß','Zurück'],
            ['.','/','@','+','-',':','{','}','<','>','Löschen'],
            ['.de','.com','Leer','.net','.tv']
        ];
        this.numberKeys = [];
        this.numberKeys[TX_KEY_0] = [[0,9]];
        this.numberKeys[TX_KEY_1] = [[0,0]];
        this.numberKeys[TX_KEY_2] = [[1,0],[1,1],[1,2],[0,1]];
        this.numberKeys[TX_KEY_3] = [[1,3],[1,4],[1,5],[0,2]];
        this.numberKeys[TX_KEY_4] = [[1,6],[1,7],[1,8],[0,3]];
        this.numberKeys[TX_KEY_5] = [[1,9],[2,0],[2,1],[0,4]];
        this.numberKeys[TX_KEY_6] = [[2,2],[2,3],[2,4],[0,5]];
        this.numberKeys[TX_KEY_7] = [[2,5],[2,6],[2,7],[2,8],[0,6]];
        this.numberKeys[TX_KEY_8] = [[2,9],[3,0],[3,1],[0,7]];
        this.numberKeys[TX_KEY_9] = [[3,2],[3,3],[3,4],[3,5],[0,8]];
        this.render();
        this.setText(this.text0);
        this.setFocus();
        this.boundOnTimer = this.onTimer.bind(this);
        this.initialized = true;
    },
    handleKey: function(key){
        switch(key){
            case TX_KEY_LEFT: this.resetNumberKey(); this.moveLeft(); return true;
            case TX_KEY_RIGHT: this.resetNumberKey(); this.moveRight(); return true;
            case TX_KEY_UP: this.resetNumberKey(); this.moveUp(); return true;
            case TX_KEY_DOWN: this.resetNumberKey(); this.moveDown(); return true;
            case TX_KEY_RED:
            case TX_KEY_YELLOW:
            case TX_KEY_GREEN:
            case TX_KEY_BLUE: return this.handleColorKey(key);
            case TX_KEY_ENTER: return this.handleEnterKey();
            case TX_KEY_0:
            case TX_KEY_1:
            case TX_KEY_2:
            case TX_KEY_3:
            case TX_KEY_4:
            case TX_KEY_5:
            case TX_KEY_6:
            case TX_KEY_7:
            case TX_KEY_8:
            case TX_KEY_9: this.handleNumberKey(key); return true;

        }
    },
    handleColorKey: function(key){
        this.resetNumberKey();
        switch(key){
            case TX_KEY_RED: this.exit(); return true;
            case TX_KEY_GREEN: this.toggleText(); return true;
            case TX_KEY_ENTER: this.enter(); return true;
            case TX_KEY_YELLOW: this.backspace(); return true;
        }
    },
    handleEnterKey: function() {
        this.resetNumberKey();
        if (this.col == 10){
            switch(this.row){
                case 0: this.exit();return true;
                case 1: this.toggleText();return true;
                case 2: this.enter();return true;
                case 3: this.backspace();return true;
                case 4: this.clear();return true;
                case 5: this.insertText();return true;
            }
        }
        if (this.row == 5 && this.col==2){
            this.input.kbdInsert(' ');
            return true;
        }
        this.insertText();
        return true;
    },
    handleNumberKey: function(key){
        if (this.lastNumberKey !== false && this.lastNumberKey !== key){
            this.insertText();
            this.resetNumberKey();
        }
        var keys = this.numberKeys[key];
        if (this.lastNumberKey === false) {
            this.numberKeyIndex = 0;
        }
        if (this.lastNumberKey === key){
            this.numberKeyIndex++;
        }
        if (this.numberKeyIndex > keys.length-1){
            this.numberKeyIndex = 0;
        }
        var temp = keys[this.numberKeyIndex];
        this.row = temp[0];
        this.col = temp[1];
        this.setFocus();
        this.lastNumberKey = key;
        this.startTimer();
    },
    onTimer: function() {
        this.resetNumberKey();
        this.insertText();
    },
    startTimer: function() {
        this.stopTimer();
        this.timer = setTimeout(this.boundOnTimer, 2000);
    },
    stopTimer: function() {
        if (this.timer){
            clearTimeout(this.timer);
        }
    },
    resetNumberKey: function() {
        this.stopTimer();
        this.lastNumberKey = false;
        this.numberKeyIndex = 0;
    },
    insertText: function() {
        this.input.kbdInsert(this.focusEl.getHtml());
    },
    show:function (input) {
        this.input = input;
        this.init();
        this.element.show();
        this.visible = true;
    },
    hide:function () {
        this.resetNumberKey();
        if(this.element && this.element.dom){
            this.element.hide();
        }
        this.visible = false;
    },
    exit: function() {
        this.resetNumberKey();
        this.input.kbdExit();
        this.fireEvent('exit');
    },
    enter: function() {
        this.resetNumberKey();
        this.input.kbdEnter();
        this.fireEvent('enter');
    },
    clear: function() {
        this.resetNumberKey();
        this.input.kbdClear();
        this.fireEvent('clear');
    },
    backspace: function() {
        this.resetNumberKey();
        this.input.kbdBackspace();
        this.fireEvent('backspace');
    },
    render: function() {
        var row, col, html=[];
        for(row=0; row < 5; row++){
            html.push('<ul id="kbd-row-'+row+'">');
            for (col = 0; col < 11; col++){
                html.push('<li id="kbd-'+row+'-'+col+'" class="kbd-col-'+col+'"></li>');
            }
            html.push('</ul>');
        }
        html.push('<ul id="kbd-row-5">');
        html.push('<li id="kbd-5-0" class="kbd-col-0"></li>');
        html.push('<li id="kbd-5-1" class="kbd-col-1"></li>');
        html.push('<li id="kbd-5-2" class="kbd-col-2"></li>');
        html.push('<li id="kbd-5-3" class="kbd-col-3"></li>');
        html.push('<li id="kbd-5-4" class="kbd-col-4"></li>');
        html.push('</ul>');
        this.element.setHtml(html);
    },
    setText: function(text){
        var row, el;
        for (var y= 0, ylen = text.length; y < ylen; y++){
            row = text[y];
            for (var x= 0, xlen = row.length; x<xlen; x++){
                el = Tx.fly('kbd-'+y+'-'+x);
                if (el && el.dom) {
                    el.setHtml(Tx.htmlspecialchars(row[x]));
                }
            }
        }
    },
    toggleText: function() {
        if (this.currentText === 0){
            this.setText(this.text1);
            this.currentText = 1;
        }
        else {
            this.setText(this.text0);
            this.currentText = 0;
        }
    },
    setFocus: function() {
        if (this.focusEl){
            this.focusEl.removeClass('focus');
        }
        this.focusEl = Tx.get('kbd-'+this.row+'-'+this.col);
        this.focusEl.addClass('focus');
    },
    moveRight: function() {
        var row = this.row, col = this.col;
        col++;
        if (col > 10){
            col = 0;
        }
        if (row == 5 && col > 4){
            col = 0;
        }
        this.col = col;
        this.setFocus();
    },
    moveDown: function() {
        var row = this.row, col = this.col;
        row++;
        if (row > 5){
            row = 0;
        }
        if (row == 5){
            switch(col){
                case 0:;case 1: col= 0; break;
                case 2:;case 3: col= 1; break;
                case 4:;case 5:;case 6:;case 7: col= 2; break;
                case 8:;case 9: col= 3; break;
                case 10: col=4; break;
            }
        }
        else if(row == 0){
            switch(col){
                case 0: col = 0; break;
                case 1: col = 2; break;
                case 2: col = 4; break;
                case 3: col = 8; break;
                case 4: col = 10; break;
            }
        }
        this.col = col;
        this.row = row;
        this.setFocus();
    },
    moveLeft: function() {
        var row = this.row, col = this.col;
        col--;
        if (col < 0) {
            col = row == 5 ? 4 : 10;
        }
        this.col = col;
        this.setFocus();
    },
    moveUp: function() {
        var row = this.row, col = this.col;
        row--;
        if (row < 0){
            row = 5;
        }
        if(row == 5){
            switch(col){
                case 0: case 1: col = 0; break;
                case 2: case 3: col = 1; break;
                case 4: case 5: case 6: case 7: col = 2; break;
                case 8: case 9: col = 3; break;
                case 10: col = 4; break;
            }
        }
        else if(row == 4){
            switch(col){
                case 0: col=0; break;
                case 1: col=2; break;
                case 2: col=4; break;
                case 3: col=8; break;
                case 4: col=10; break;
            }
        }
        this.col = col;
        this.row = row;
        this.setFocus();
    },
    handleMouseOver: function(event) {
        var target = event.target;
        while (target && target.nodeType == 3) target = target.parentNode;
        if (target && target.tagName.toLowerCase() == 'li'){
            var id = target.id, vals = id.split('-'), row = vals[1], col = vals[2];
            this.col = Number(col);
            this.row = Number(row);
            this.setFocus();
        }
    },
    handleMouseClick: function(event) {
        this.handleKey(TX_KEY_ENTER);
    }

}));
