/*!
 * Sizzle CSS Selector Engine
 *  Copyright 2011, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^\[\]]*\]|['"][^'"]*['"]|[^\[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g,
	expando = "sizcache" + (Math.random() + '').replace('.', ''),
	done = 0,
	toString = Object.prototype.toString,
	hasDuplicate = false,
	baseHasDuplicate = true,
	rBackslash = /\\/g,
	rReturn = /\r\n/g,
	rNonWord = /\W/;

// Here we check if the JavaScript engine is using some sort of
// optimization where it does not always call our comparision
// function. If that is the case, discard the hasDuplicate value.
//   Thus far that includes Google Chrome.
[0, 0].sort(function() {
	baseHasDuplicate = false;
	return 0;
});

var Sizzle = function( selector, context, results, seed ) {
	results = results || [];
	context = context || document;

	var origContext = context;

	if ( context.nodeType !== 1 && context.nodeType !== 9 ) {
		return [];
	}

	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var m, set, checkSet, extra, ret, cur, pop, i,
		prune = true,
		contextXML = Sizzle.isXML( context ),
		parts = [],
		soFar = selector;

	// Reset the position of the chunker regexp (start from head)
	do {
		chunker.exec( "" );
		m = chunker.exec( soFar );

		if ( m ) {
			soFar = m[3];

			parts.push( m[1] );

			if ( m[2] ) {
				extra = m[3];
				break;
			}
		}
	} while ( m );

	if ( parts.length > 1 && origPOS.exec( selector ) ) {

		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context, seed );

		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] ) {
					selector += parts.shift();
				}

				set = posProcess( selector, set, seed );
			}
		}

	} else {
		// Take a shortcut and set the context if the root selector is an ID
		// (but not if it'll be faster if the inner selector is an ID)
		if ( !seed && parts.length > 1 && context.nodeType === 9 && !contextXML &&
				Expr.match.ID.test(parts[0]) && !Expr.match.ID.test(parts[parts.length - 1]) ) {

			ret = Sizzle.find( parts.shift(), context, contextXML );
			context = ret.expr ?
				Sizzle.filter( ret.expr, ret.set )[0] :
				ret.set[0];
		}

		if ( context ) {
			ret = seed ?
				{ expr: parts.pop(), set: makeArray(seed) } :
				Sizzle.find( parts.pop(), parts.length === 1 && (parts[0] === "~" || parts[0] === "+") && context.parentNode ? context.parentNode : context, contextXML );

			set = ret.expr ?
				Sizzle.filter( ret.expr, ret.set ) :
				ret.set;

			if ( parts.length > 0 ) {
				checkSet = makeArray( set );

			} else {
				prune = false;
			}

			while ( parts.length ) {
				cur = parts.pop();
				pop = cur;

				if ( !Expr.relative[ cur ] ) {
					cur = "";
				} else {
					pop = parts.pop();
				}

				if ( pop == null ) {
					pop = context;
				}

				Expr.relative[ cur ]( checkSet, pop, contextXML );
			}

		} else {
			checkSet = parts = [];
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		Sizzle.error( cur || selector );
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );

		} else if ( context && context.nodeType === 1 ) {
			for ( i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && Sizzle.contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}

		} else {
			for ( i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}

	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, origContext, results, seed );
		Sizzle.uniqueSort( results );
	}

	return results;
};

Sizzle.uniqueSort = function( results ) {
	if ( sortOrder ) {
		hasDuplicate = baseHasDuplicate;
		results.sort( sortOrder );

		if ( hasDuplicate ) {
			for ( var i = 1; i < results.length; i++ ) {
				if ( results[i] === results[ i - 1 ] ) {
					results.splice( i--, 1 );
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function( expr, set ) {
	return Sizzle( expr, null, null, set );
};

Sizzle.matchesSelector = function( node, expr ) {
	return Sizzle( expr, null, null, [node] ).length > 0;
};

Sizzle.find = function( expr, context, isXML ) {
	var set, i, len, match, type, left;

	if ( !expr ) {
		return [];
	}

	for ( i = 0, len = Expr.order.length; i < len; i++ ) {
		type = Expr.order[i];

		if ( (match = Expr.leftMatch[ type ].exec( expr )) ) {
			left = match[1];
			match.splice( 1, 1 );

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace( rBackslash, "" );
				set = Expr.find[ type ]( match, context, isXML );

				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = typeof context.getElementsByTagName !== "undefined" ?
			context.getElementsByTagName( "*" ) :
			[];
	}

	return { set: set, expr: expr };
};

Sizzle.filter = function( expr, set, inplace, not ) {
	var match, anyFound,
		type, found, item, filter, left,
		i, pass,
		old = expr,
		result = [],
		curLoop = set,
		isXMLFilter = set && set[0] && Sizzle.isXML( set[0] );

	while ( expr && set.length ) {
		for ( type in Expr.filter ) {
			if ( (match = Expr.leftMatch[ type ].exec( expr )) != null && match[2] ) {
				filter = Expr.filter[ type ];
				left = match[1];

				anyFound = false;

				match.splice(1,1);

				if ( left.substr( left.length - 1 ) === "\\" ) {
					continue;
				}

				if ( curLoop === result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;

					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							pass = not ^ found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;

								} else {
									curLoop[i] = false;
								}

							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		// Improper expression
		if ( expr === old ) {
			if ( anyFound == null ) {
				Sizzle.error( expr );

			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Utility function for retreiving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
var getText = Sizzle.getText = function( elem ) {
    var i, node,
		nodeType = elem.nodeType,
		ret = "";

	if ( nodeType ) {
		if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
			// Use textContent || innerText for elements
			if ( typeof elem.textContent === 'string' ) {
				return elem.textContent;
			} else if ( typeof elem.innerText === 'string' ) {
				// Replace IE's carriage returns
				return elem.innerText.replace( rReturn, '' );
			} else {
				// Traverse it's children
				for ( elem = elem.firstChild; elem; elem = elem.nextSibling) {
					ret += getText( elem );
				}
			}
		} else if ( nodeType === 3 || nodeType === 4 ) {
			return elem.nodeValue;
		}
	} else {

		// If no nodeType, this is expected to be an array
		for ( i = 0; (node = elem[i]); i++ ) {
			// Do not traverse comment nodes
			if ( node.nodeType !== 8 ) {
				ret += getText( node );
			}
		}
	}
	return ret;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],

	match: {
		ID: /#((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF\-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF\-]|\\.)+)\s*(?:(\S?=)\s*(?:(['"])(.*?)\3|(#?(?:[\w\u00c0-\uFFFF\-]|\\.)*)|)|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*\-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\(\s*(even|odd|(?:[+\-]?\d+|(?:[+\-]?\d*)?n\s*(?:[+\-]\s*\d+)?))\s*\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^\-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF\-]|\\.)+)(?:\((['"]?)((?:\([^\)]+\)|[^\(\)]*)+)\2\))?/
	},

	leftMatch: {},

	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},

	attrHandle: {
		href: function( elem ) {
			return elem.getAttribute( "href" );
		},
		type: function( elem ) {
			return elem.getAttribute( "type" );
		}
	},

	relative: {
		"+": function(checkSet, part){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !rNonWord.test( part ),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag ) {
				part = part.toLowerCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName.toLowerCase() === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},

		">": function( checkSet, part ) {
			var elem,
				isPartStr = typeof part === "string",
				i = 0,
				l = checkSet.length;

			if ( isPartStr && !rNonWord.test( part ) ) {
				part = part.toLowerCase();

				for ( ; i < l; i++ ) {
					elem = checkSet[i];

					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName.toLowerCase() === part ? parent : false;
					}
				}

			} else {
				for ( ; i < l; i++ ) {
					elem = checkSet[i];

					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},

		"": function(checkSet, part, isXML){
			var nodeCheck,
				doneName = done++,
				checkFn = dirCheck;

			if ( typeof part === "string" && !rNonWord.test( part ) ) {
				part = part.toLowerCase();
				nodeCheck = part;
				checkFn = dirNodeCheck;
			}

			checkFn( "parentNode", part, doneName, checkSet, nodeCheck, isXML );
		},

		"~": function( checkSet, part, isXML ) {
			var nodeCheck,
				doneName = done++,
				checkFn = dirCheck;

			if ( typeof part === "string" && !rNonWord.test( part ) ) {
				part = part.toLowerCase();
				nodeCheck = part;
				checkFn = dirNodeCheck;
			}

			checkFn( "previousSibling", part, doneName, checkSet, nodeCheck, isXML );
		}
	},

	find: {
		ID: function( match, context, isXML ) {
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				// Check parentNode to catch when Blackberry 4.6 returns
				// nodes that are no longer in the document #6963
				return m && m.parentNode ? [m] : [];
			}
		},

		NAME: function( match, context ) {
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [],
					results = context.getElementsByName( match[1] );

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},

		TAG: function( match, context ) {
			if ( typeof context.getElementsByTagName !== "undefined" ) {
				return context.getElementsByTagName( match[1] );
			}
		}
	},
	preFilter: {
		CLASS: function( match, curLoop, inplace, result, not, isXML ) {
			match = " " + match[1].replace( rBackslash, "" ) + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").replace(/[\t\n\r]/g, " ").indexOf(match) >= 0) ) {
						if ( !inplace ) {
							result.push( elem );
						}

					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},

		ID: function( match ) {
			return match[1].replace( rBackslash, "" );
		},

		TAG: function( match, curLoop ) {
			return match[1].replace( rBackslash, "" ).toLowerCase();
		},

		CHILD: function( match ) {
			if ( match[1] === "nth" ) {
				if ( !match[2] ) {
					Sizzle.error( match[0] );
				}

				match[2] = match[2].replace(/^\+|\s*/g, '');

				// parse equations like 'even', 'odd', '5', '2n', '3n+2', '4n-1', '-n+6'
				var test = /(-?)(\d*)(?:n([+\-]?\d*))?/.exec(
					match[2] === "even" && "2n" || match[2] === "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				// calculate the numbers (first)n+(last) including if they are negative
				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}
			else if ( match[2] ) {
				Sizzle.error( match[0] );
			}

			// TODO: Move to normal caching system
			match[0] = done++;

			return match;
		},

		ATTR: function( match, curLoop, inplace, result, not, isXML ) {
			var name = match[1] = match[1].replace( rBackslash, "" );

			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			// Handle if an un-quoted value was used
			match[4] = ( match[4] || match[5] || "" ).replace( rBackslash, "" );

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},

		PSEUDO: function( match, curLoop, inplace, result, not ) {
			if ( match[1] === "not" ) {
				// If we're dealing with a complex expression, or a simple one
				if ( ( chunker.exec(match[3]) || "" ).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);

				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);

					if ( !inplace ) {
						result.push.apply( result, ret );
					}

					return false;
				}

			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}

			return match;
		},

		POS: function( match ) {
			match.unshift( true );

			return match;
		}
	},

	filters: {
		enabled: function( elem ) {
			return elem.disabled === false && elem.type !== "hidden";
		},

		disabled: function( elem ) {
			return elem.disabled === true;
		},

		checked: function( elem ) {
			return elem.checked === true;
		},

		selected: function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		parent: function( elem ) {
			return !!elem.firstChild;
		},

		empty: function( elem ) {
			return !elem.firstChild;
		},

		has: function( elem, i, match ) {
			return !!Sizzle( match[3], elem ).length;
		},

		header: function( elem ) {
			return (/h\d/i).test( elem.nodeName );
		},

		text: function( elem ) {
			var attr = elem.getAttribute( "type" ), type = elem.type;
			// IE6 and 7 will map elem.type to 'text' for new HTML5 types (search, etc)
			// use getAttribute instead to test this case
			return elem.nodeName.toLowerCase() === "input" && "text" === type && ( attr === type || attr === null );
		},

		radio: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "radio" === elem.type;
		},

		checkbox: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "checkbox" === elem.type;
		},

		file: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "file" === elem.type;
		},

		password: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "password" === elem.type;
		},

		submit: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return (name === "input" || name === "button") && "submit" === elem.type;
		},

		image: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "image" === elem.type;
		},

		reset: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return (name === "input" || name === "button") && "reset" === elem.type;
		},

		button: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && "button" === elem.type || name === "button";
		},

		input: function( elem ) {
			return (/input|select|textarea|button/i).test( elem.nodeName );
		},

		focus: function( elem ) {
			return elem === elem.ownerDocument.activeElement;
		}
	},
	setFilters: {
		first: function( elem, i ) {
			return i === 0;
		},

		last: function( elem, i, match, array ) {
			return i === array.length - 1;
		},

		even: function( elem, i ) {
			return i % 2 === 0;
		},

		odd: function( elem, i ) {
			return i % 2 === 1;
		},

		lt: function( elem, i, match ) {
			return i < match[3] - 0;
		},

		gt: function( elem, i, match ) {
			return i > match[3] - 0;
		},

		nth: function( elem, i, match ) {
			return match[3] - 0 === i;
		},

		eq: function( elem, i, match ) {
			return match[3] - 0 === i;
		}
	},
	filter: {
		PSEUDO: function( elem, match, i, array ) {
			var name = match[1],
				filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );

			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || getText([ elem ]) || "").indexOf(match[3]) >= 0;

			} else if ( name === "not" ) {
				var not = match[3];

				for ( var j = 0, l = not.length; j < l; j++ ) {
					if ( not[j] === elem ) {
						return false;
					}
				}

				return true;

			} else {
				Sizzle.error( name );
			}
		},

		CHILD: function( elem, match ) {
			var first, last,
				doneName, parent, cache,
				count, diff,
				type = match[1],
				node = elem;

			switch ( type ) {
				case "only":
				case "first":
					while ( (node = node.previousSibling) ) {
						if ( node.nodeType === 1 ) {
							return false;
						}
					}

					if ( type === "first" ) {
						return true;
					}

					node = elem;

					/* falls through */
				case "last":
					while ( (node = node.nextSibling) ) {
						if ( node.nodeType === 1 ) {
							return false;
						}
					}

					return true;

				case "nth":
					first = match[2];
					last = match[3];

					if ( first === 1 && last === 0 ) {
						return true;
					}

					doneName = match[0];
					parent = elem.parentNode;

					if ( parent && (parent[ expando ] !== doneName || !elem.nodeIndex) ) {
						count = 0;

						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						}

						parent[ expando ] = doneName;
					}

					diff = elem.nodeIndex - last;

					if ( first === 0 ) {
						return diff === 0;

					} else {
						return ( diff % first === 0 && diff / first >= 0 );
					}
			}
		},

		ID: function( elem, match ) {
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},

		TAG: function( elem, match ) {
			return (match === "*" && elem.nodeType === 1) || !!elem.nodeName && elem.nodeName.toLowerCase() === match;
		},

		CLASS: function( elem, match ) {
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},

		ATTR: function( elem, match ) {
			var name = match[1],
				result = Sizzle.attr ?
					Sizzle.attr( elem, name ) :
					Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				!type && Sizzle.attr ?
				result != null :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value !== check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},

		POS: function( elem, match, i, array ) {
			var name = match[2],
				filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS,
	fescape = function(all, num){
		return "\\" + (num - 0 + 1);
	};

for ( var type in Expr.match ) {
	Expr.match[ type ] = new RegExp( Expr.match[ type ].source + (/(?![^\[]*\])(?![^\(]*\))/.source) );
	Expr.leftMatch[ type ] = new RegExp( /(^(?:.|\r|\n)*?)/.source + Expr.match[ type ].source.replace(/\\(\d+)/g, fescape) );
}
// Expose origPOS
// "global" as in regardless of relation to brackets/parens
Expr.match.globalPOS = origPOS;

var makeArray = function( array, results ) {
	array = Array.prototype.slice.call( array, 0 );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}

	return array;
};

// Perform a simple check to determine if the browser is capable of
// converting a NodeList to an array using builtin methods.
// Also verifies that the returned array holds DOM nodes
// (which is not the case in the Blackberry browser)
try {
	Array.prototype.slice.call( document.documentElement.childNodes, 0 )[0].nodeType;

// Provide a fallback method if it does not work
} catch( e ) {
	makeArray = function( array, results ) {
		var i = 0,
			ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );

		} else {
			if ( typeof array.length === "number" ) {
				for ( var l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}

			} else {
				for ( ; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder, siblingCheck;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		if ( !a.compareDocumentPosition || !b.compareDocumentPosition ) {
			return a.compareDocumentPosition ? -1 : 1;
		}

		return a.compareDocumentPosition(b) & 4 ? -1 : 1;
	};

} else {
	sortOrder = function( a, b ) {
		// The nodes are identical, we can exit early
		if ( a === b ) {
			hasDuplicate = true;
			return 0;

		// Fallback to using sourceIndex (in IE) if it's available on both nodes
		} else if ( a.sourceIndex && b.sourceIndex ) {
			return a.sourceIndex - b.sourceIndex;
		}

		var al, bl,
			ap = [],
			bp = [],
			aup = a.parentNode,
			bup = b.parentNode,
			cur = aup;

		// If the nodes are siblings (or identical) we can do a quick check
		if ( aup === bup ) {
			return siblingCheck( a, b );

		// If no parents were found then the nodes are disconnected
		} else if ( !aup ) {
			return -1;

		} else if ( !bup ) {
			return 1;
		}

		// Otherwise they're somewhere else in the tree so we need
		// to build up a full list of the parentNodes for comparison
		while ( cur ) {
			ap.unshift( cur );
			cur = cur.parentNode;
		}

		cur = bup;

		while ( cur ) {
			bp.unshift( cur );
			cur = cur.parentNode;
		}

		al = ap.length;
		bl = bp.length;

		// Start walking down the tree looking for a discrepancy
		for ( var i = 0; i < al && i < bl; i++ ) {
			if ( ap[i] !== bp[i] ) {
				return siblingCheck( ap[i], bp[i] );
			}
		}

		// We ended someplace up the tree so do a sibling check
		return i === al ?
			siblingCheck( a, bp[i], -1 ) :
			siblingCheck( ap[i], b, 1 );
	};

	siblingCheck = function( a, b, ret ) {
		if ( a === b ) {
			return ret;
		}

		var cur = a.nextSibling;

		while ( cur ) {
			if ( cur === b ) {
				return -1;
			}

			cur = cur.nextSibling;
		}

		return 1;
	};
}

// Check to see if the browser returns elements by name when
// querying by getElementById (and provide a workaround)
(function(){
	// We're going to inject a fake input element with a specified name
	var form = document.createElement("div"),
		id = "script" + (new Date()).getTime(),
		root = document.documentElement;

	form.innerHTML = "<a name='" + id + "'/>";

	// Inject it into the root element, check its status, and remove it quickly
	root.insertBefore( form, root.firstChild );

	// The workaround has to do additional checks after a getElementById
	// Which slows things down for other browsers (hence the branching)
	if ( document.getElementById( id ) ) {
		Expr.find.ID = function( match, context, isXML ) {
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);

				return m ?
					m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ?
						[m] :
						undefined :
					[];
			}
		};

		Expr.filter.ID = function( elem, match ) {
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");

			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );

	// release memory in IE
	root = form = null;
})();

(function(){
	// Check to see if the browser returns only elements
	// when doing getElementsByTagName("*")

	// Create a fake element
	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	// Make sure no comments are found
	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function( match, context ) {
			var results = context.getElementsByTagName( match[1] );

			// Filter out possible comments
			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	// Check to see if an attribute returns normalized href attributes
	div.innerHTML = "<a href='#'></a>";

	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {

		Expr.attrHandle.href = function( elem ) {
			return elem.getAttribute( "href", 2 );
		};
	}

	// release memory in IE
	div = null;
})();

if ( document.querySelectorAll ) {
	(function(){
		var oldSizzle = Sizzle,
			div = document.createElement("div"),
			id = "__sizzle__";

		div.innerHTML = "<p class='TEST'></p>";

		// Safari can't handle uppercase or unicode characters when
		// in quirks mode.
		if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
			return;
		}

		Sizzle = function( query, context, extra, seed ) {
			context = context || document;

			// Only use querySelectorAll on non-XML documents
			// (ID selectors don't work in non-HTML documents)
			if ( !seed && !Sizzle.isXML(context) ) {
				// See if we find a selector to speed up
				var match = /^(\w+$)|^\.([\w\-]+$)|^#([\w\-]+$)/.exec( query );

				if ( match && (context.nodeType === 1 || context.nodeType === 9) ) {
					// Speed-up: Sizzle("TAG")
					if ( match[1] ) {
						return makeArray( context.getElementsByTagName( query ), extra );

					// Speed-up: Sizzle(".CLASS")
					} else if ( match[2] && Expr.find.CLASS && context.getElementsByClassName ) {
						return makeArray( context.getElementsByClassName( match[2] ), extra );
					}
				}

				if ( context.nodeType === 9 ) {
					// Speed-up: Sizzle("body")
					// The body element only exists once, optimize finding it
					if ( query === "body" && context.body ) {
						return makeArray( [ context.body ], extra );

					// Speed-up: Sizzle("#ID")
					} else if ( match && match[3] ) {
						var elem = context.getElementById( match[3] );

						// Check parentNode to catch when Blackberry 4.6 returns
						// nodes that are no longer in the document #6963
						if ( elem && elem.parentNode ) {
							// Handle the case where IE and Opera return items
							// by name instead of ID
							if ( elem.id === match[3] ) {
								return makeArray( [ elem ], extra );
							}

						} else {
							return makeArray( [], extra );
						}
					}

					try {
						return makeArray( context.querySelectorAll(query), extra );
					} catch(qsaError) {}

				// qSA works strangely on Element-rooted queries
				// We can work around this by specifying an extra ID on the root
				// and working up from there (Thanks to Andrew Dupont for the technique)
				// IE 8 doesn't work on object elements
				} else if ( context.nodeType === 1 && context.nodeName.toLowerCase() !== "object" ) {
					var oldContext = context,
						old = context.getAttribute( "id" ),
						nid = old || id,
						hasParent = context.parentNode,
						relativeHierarchySelector = /^\s*[+~]/.test( query );

					if ( !old ) {
						context.setAttribute( "id", nid );
					} else {
						nid = nid.replace( /'/g, "\\$&" );
					}
					if ( relativeHierarchySelector && hasParent ) {
						context = context.parentNode;
					}

					try {
						if ( !relativeHierarchySelector || hasParent ) {
							return makeArray( context.querySelectorAll( "[id='" + nid + "'] " + query ), extra );
						}

					} catch(pseudoError) {
					} finally {
						if ( !old ) {
							oldContext.removeAttribute( "id" );
						}
					}
				}
			}

			return oldSizzle(query, context, extra, seed);
		};

		for ( var prop in oldSizzle ) {
			Sizzle[ prop ] = oldSizzle[ prop ];
		}

		// release memory in IE
		div = null;
	})();
}

(function(){
	var html = document.documentElement,
		matches = html.matchesSelector || html.mozMatchesSelector || html.webkitMatchesSelector || html.msMatchesSelector;

	if ( matches ) {
		// Check to see if it's possible to do matchesSelector
		// on a disconnected node (IE 9 fails this)
		var disconnectedMatch = !matches.call( document.createElement( "div" ), "div" ),
			pseudoWorks = false;

		try {
			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( document.documentElement, "[test!='']:sizzle" );

		} catch( pseudoError ) {
			pseudoWorks = true;
		}

		Sizzle.matchesSelector = function( node, expr ) {
			// Make sure that attribute selectors are quoted
			expr = expr.replace(/\=\s*([^'"\]]*)\s*\]/g, "='$1']");

			if ( !Sizzle.isXML( node ) ) {
				try {
					if ( pseudoWorks || !Expr.match.PSEUDO.test( expr ) && !/!=/.test( expr ) ) {
						var ret = matches.call( node, expr );

						// IE 9's matchesSelector returns false on disconnected nodes
						if ( ret || !disconnectedMatch ||
								// As well, disconnected nodes are said to be in a document
								// fragment in IE 9, so check for that
								node.document && node.document.nodeType !== 11 ) {
							return ret;
						}
					}
				} catch(e) {}
			}

			return Sizzle(expr, null, null, [node]).length > 0;
		};
	}
})();

(function(){
	var div = document.createElement("div");

	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	// Opera can't find a second classname (in 9.6)
	// Also, make sure that getElementsByClassName actually exists
	if ( !div.getElementsByClassName || div.getElementsByClassName("e").length === 0 ) {
		return;
	}

	// Safari caches class attributes, doesn't catch changes (in 3.2)
	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 ) {
		return;
	}

	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function( match, context, isXML ) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};

	// release memory in IE
	div = null;
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];

		if ( elem ) {
			var match = false;

			elem = elem[dir];

			while ( elem ) {
				if ( elem[ expando ] === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem[ expando ] = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName.toLowerCase() === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];

		if ( elem ) {
			var match = false;

			elem = elem[dir];

			while ( elem ) {
				if ( elem[ expando ] === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem[ expando ] = doneName;
						elem.sizset = i;
					}

					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

if ( document.documentElement.contains ) {
	Sizzle.contains = function( a, b ) {
		return a !== b && (a.contains ? a.contains(b) : true);
	};

} else if ( document.documentElement.compareDocumentPosition ) {
	Sizzle.contains = function( a, b ) {
		return !!(a.compareDocumentPosition(b) & 16);
	};

} else {
	Sizzle.contains = function() {
		return false;
	};
}

Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = (elem ? elem.ownerDocument || elem : 0).documentElement;

	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

var posProcess = function( selector, context, seed ) {
	var match,
		tmpSet = [],
		later = "",
		root = context.nodeType ? [context] : context;

	// Position selectors must be done after the filter
	// And so must :not(positional) so we move all PSEUDOs to the end
	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet, seed );
	}

	return Sizzle.filter( later, tmpSet );
};

// EXPOSE

window.Sizzle = Sizzle;

})();
/**
 * Tv Library.
 * @author: kasi
 * @namespace
 */

var Tx = {
    _id: 1,
    readyEvents:[],
    debugging:false,
    focusClass: 'focus',
    focusEl: null,
    /**
     * Kopiert Properties von einem Object in ein anderes
     * @param {object} obj Das Ziel Objekt
     * @param {object} src Das Source Objekt
     * @returns {object} Das erweiterte Object
     */
    apply: function(obj, src){
        if (!src){
            return obj;
        }
        for(var p in src){
            obj[p] = src[p];
        }
        return obj;
    },
    /**
     * Wie Tx.apply, kopiert aber nur Properties, die im Ziel Objekt nicht existieren
     * @see Tx.apply
     * @param {object} obj Das Ziel Objekt
     * @param {object} src Das Source Objekt
     * @returns {object} Das erweiterte Object
     */
    applyIf: function(obj, src){
        if (!src){
            return obj;
        }
        for(var p in src){
            if (typeof obj[p] === 'undefined'){
                obj[p] = src[p];
            }
        }
        return obj;
    },
    /**
     * Liefert ein Tx.Element anhand der ID
     * @see Tx.Element
     * @param {string|Tx.Element} id
     * @returns Tx.Element | null
     */
    get: function(id, initState) {
        if (id && id.dom){
            return id;
        }
        var result =  new Tx.Element(id, initState);
        return result.dom ? result : null;
    },
    /**
     * Liefert das Tx.Element das in cls angegeben wird
     * @param {Function}  cls
     * @param {string|Tx.Element} id
     * @param {Object} options
     * @return Tx.Element
     */
    getElement: function(cls, id, options){
        var el = new cls(id.dom || id, options);
        el.initState();
        return el;
    },
    /**
     * Liefert ein gemeinsam genutztes Tx.Element anhand der ID. Nicht zum permaneten Speichern verwenden
     * @see Tx.Element
     * @param {string|Tx.Element} id
     * @returns Tx.Element | null
     */
    fly: function(id){
        if (!id) return null;
        if (id.dom){
            return id;
        }
        this._fly.dom = id.tagName ? id : document.getElementById(id);
        return this._fly.dom ? this._fly : null;
    },
    /**
     * Kurzform für document.getElementById(id)
     * @param {string} id
     * @returns HTMLElement
     */
    byId: function(id){
        return document.getElementById(id);
    },
    /**
     * Tx.Element Liste
     * selector ist ein einfacher selector (div.classname)
     * base ist der Kontext (optional)
     * @see Tx.Elements
     * @param {string} selector
     * @param {HTMLElement} [base=document]
     * @returns Tx.Elements
     */
    query: function(selector, base){
        var els = new Tx.Elements();
        els.add(selector, base);
        return els;
    },
    /**
     * Prüft ob ein Element mit dem einfachen Selector übereinstimmt
     * @param {HTMLElement} el
     * @param {string} selector
     * @returns boolean
     */
    selectorMatch: function(el, selector){
        var tagName, className, tmp;
        el = el.dom || el;
        if (!el.tagName){
            return false;
        }
        tmp = selector.split('.');
        tagName = (tmp[0] || '*').toUpperCase();
        className = tmp[0];
        if (tagName == '*' || el.tagName.toUpperCase() == tagName) {
            if (className){
                return className.toUpperCase() == el.className.toUpperCase();
            }
            else {
                return true;
            }
        }
        else {
            return false;
        }

    },
    /**
     * Wandelt die Zeichen <pre><, >, & in &lt; , &gt; und &amp;</pre>
     * @param {string} str
     * @returns {string}
     */
    htmlspecialchars: function(str){
        str = String(str);
        return str.replace(/([&<>])(amp;)?/g,function(all,  chr, extra){
            switch(chr){
                case '>': return '&gt;'+(extra||'');
                case '<': return '&lt;'+(extra||'');
                case '&': return '&amp;';
            }
        });
    },
    /**
     * Wandelt deutsche HTMl Entities in UTF-8 Zeichen
     * @param {string} str
     * @returns {string}
     */
    htmldecodeEntity: function(str){
        str = String(str);
        return str.replace(/&([a-z0-9]{2,6});/gi, function(all, name){
            switch(name){
                case 'auml': return 'ä';
                case 'Auml': return 'Ä';
                case 'ouml': return 'ö';
                case 'Ouml': return 'Ö';
                case 'uuml': return 'ü';
                case 'Uuml': return 'Ü';
                case 'szlig': return 'ß';
                case 'euro': return '€';
                case 'frac14': return '¼';
                case 'frac12': return '½';
                case 'frac34': return '¾';
                case 'nbsp': return ' ';
            }
        });
    },
    /**
     * Ermittelt den Type eines Wertes
     * @param value
     * @returns {string} string|number|boolean|function|undefined|object|element|attribute|text|array|date
     */
    typeOf: function(value){
        if (value === null){
            return 'null';
        }
        var type = typeof value;
        switch (typeof value){
            case 'string':
            case 'number':
            case 'boolean':
            case 'function':
            case 'undefined':
                return type;
            case 'object':
                try {
                    if (value.txType){
                        return value.txType;
                    }
                    switch (value.nodeType){
                        case 1: return 'element';
                        case 2: return 'attribute';
                        case 3: return 'text'
                    }
                }
                catch(e){}
                return 'object';
            break;
            default:
                return 'undefined';
        }
    },
    /**
     * Erzeugt eine neue Klasse, die von einer Basisklasse abgeleitet wird
     * @param {object} base
     * @param {object} def
     * @returns {function} Die neue Klasse
     */
    extend: function(base, def){
        if (def && (!def.constructor || def.constructor === Object.prototype.constructor)){
            def.constructor = function(){
                base.prototype.constructor.apply(this, arguments);
            }
        }
        var F = function(){}, cls = def.constructor || function(){};
        F.prototype = base.prototype;
        cls.prototype = new F();
        Tx.apply(cls.prototype, def);
        return cls;
    },
    staticClass: function(base, def){
        return new (Tx.extend(base, def));
    },
    /**
     * Erzeugt eine neue Id. Wenn ein Objekt übergeben, wird das id Property gesetzt
     * @param {object} [obj]
     * @returns {int|object}
     */
    id: function(obj){
        if (obj && !obj.id){
            obj.id = 'TX'+(Tx._id++);
            return obj;
        }
        else {
            return (Tx._id++);
        }
    },
    /**
     * @returns {HTMLElement} document.body
     */
    getBody: function() {
        return document.body || document.getElementsByTagName('body')[0];
    },
    /**
     * @returns {HTMLElement} head
     */
    getHead: function() {
        return document.getElementsByTagName('head')[0];
    },
    /**
     * Fügt ein neues script Tag im head ein.
     * Wenn ein String übergeben wird ist es die Url zu einer Javascript Datei
     * Wenn ein Objekt übergeben wird sind es die Eigenschaften des Tags
     * @param {string|object} attribs
     */
    addScript: function(attribs) {
        var script = document.createElement('script');
        if (typeof attribs == 'string'){
            attribs = {
                src: attribs
            }
        }
        Tx.applyIf(attribs,{
            type: 'text/javascript',
            language: 'javascript'
        });
        for (var k in attribs){
            script.setAttribute(k, attribs[k]);
        }
        this.getHead().appendChild(script);
    },
    /**
     * Kurz für Tx.apply(obj.prototype, def)
     * @param {object} obj
     * @param {object} defs
     */
    override: function(obj, defs){
        return Tx.apply(obj.prototype, defs)
    },
    hasClass: function(el, className){
        return Tx.fly(el).className.contains(className, ' ');
    },
    getObjectFromPath: function(obj){
        var result = obj, i, l;
        for (i=1, l=arguments.length; i<l; i++){
            if (!result){
                return null;
            }
            result = result[arguments[i]];
        }
        return result;
    },
    _log: function(type, args) {
        if (!Tx.debugging){
            return;
        }
        var data = [type], i, l;
        for (i=0, l=args.length; i<l; i++){
            data.push(args[i]);
        }
        Tx.Debug.debug.apply(Tx.Debug, data);
    },
    log: function(message){
        if (!Tx.debugging) return;
        Tx._log('log', arguments);
    },
    logScreen: function(message){
        if (!Tx.debugging) return;
        Tx.Debug.log.apply(message);
    },
    debug: function() {
        if (!Tx.debugging) return;
        Tx._log('debug', arguments);
    },
    warn: function() {
        if (!Tx.debugging) return;
        Tx._log('warn', arguments);
    },
    error: function() {
        if (!Tx.debugging) return;
        Tx._log('error', arguments);
    },
    info: function() {
        if (!Tx.debugging) return;
        Tx._log('info', arguments);
    },
    exception: function() {
        if (!Tx.debugging) return;
        Tx._log('exception', arguments);
    },
    dir: function() {
        if (!Tx.debugging) return;
        Tx._log('dir', arguments);
    },
    focus: function(element){
        Tx.blur();
        Tx.focusEl = Tx.fly(element).addClass(Tx.focusClass).dom;
    },
    blur: function() {
        if (Tx.focusEl){
            Tx.fly(Tx.focusEl).removeClass(Tx.focusClass);
        }
    },
    handleMouseOverEvent: function(event){
        if (this.tagName){
            Tx.focus(this);
        }
        if (event && event.currentTarget){
            Tx.focus(event.currentTarget);
        }
    },
    handleMouseOutEvent: function(event){
        Tx.blur();
    },
    stopEvent: function(event){
        //Tx.log('stop event '+(typeof event)+' '+typeof window.event);
        if (event && event.stopPropagation){
            //Tx.log('event stopped');
            event.stopPropagation();
            event.preventDefault();
        }
    },
    addEvent: function(obj, name, fn, scope, args){
        if (obj.addEventListener){
            obj.addEventListener(name, scope ? fn = fn.bind(scope, args) : fn, false);
        }
        else if (obj.addEvent){
            obj.addEvent(name, fn, scope, args);
        }
        return fn;
    },
    reload: function(force) {
        var location = document.location && document.location.reload ? document.location : window.location;
        if (!force){
            location.reload();
        }
        else {
            var href = location.href, t = (new Date().getTime());
            if (href.indexOf('?') == -1){
                href = href +'?t='+t;
            }
            else {
                if (href.indexOf('t=') == -1){
                    href = href + '&t=' + t
                }
                else {
                    href = href.replace(/t=\d+/,'t='+t);
                }
            }
            location.href = href;
        }
    },
    historyBack: function() {
        if (document.location && document.location.history){
            document.location.history.back();
        }
        else {
            window.history.back();
        }
    },
    /**
     * Formatiert den Template-String mit den Werten aus data
     * <pre><code>
     *      Tx.format('{name] ist {age} Jahre alt',{name:'Tim',age:10})
     * </code></pre>
     * @param tpl
     * @param data
     * @param regexp
     */
    format: function(tpl, data, regexp){
        if (Tx.typeOf(tpl) == 'array'){
            tpl = tpl.join('');
        }
		return String(tpl).replace(regexp || (/\\?\{([^{}]+)\}/g), function(match, name){
			if (match.charAt(0) == '\\') return match.slice(1);
			return (data[name] != undefined) ? data[name] : '';
		});

    },
    /**
     * Create a cookie with the specified name and value. Additional settings
     * for the cookie may be optionally specified (for example: expiration,
     * access restriction, SSL).
     * @param {String} name The name of the cookie to set.
     * @param {Object} value The value to set for the cookie.
     * @param {Object} expires (Optional) Specify an expiration date the
     * cookie is to persist until.  Note that the specified Date object will
     * be converted to Greenwich Mean Time (GMT).
     * @param {String} path (Optional) Setting a path on the cookie restricts
     * access to pages that match that path. Defaults to all pages (<tt>'/'</tt>).
     * @param {String} domain (Optional) Setting a domain restricts access to
     * pages on a given domain (typically used to allow cookie access across
     * subdomains). For example, "sencha.com" will create a cookie that can be
     * accessed from any subdomain of sencha.com, including www.sencha.com,
     * support.sencha.com, etc.
     * @param {Boolean} secure (Optional) Specify true to indicate that the cookie
     * should only be accessible via SSL on a page using the HTTPS protocol.
     * Defaults to <tt>false</tt>. Note that this will only work if the page
     * calling this code uses the HTTPS protocol, otherwise the cookie will be
     * created with default options.
     */
    setCookie : function(name, value){
        var argv = arguments,
            argc = arguments.length,
            expires = (argc > 2) ? argv[2] : null,
            path = (argc > 3) ? argv[3] : '/',
            domain = (argc > 4) ? argv[4] : null,
            secure = (argc > 5) ? argv[5] : false;

        document.cookie = name + "=" + encodeURIComponent(value) + ((expires === null) ? "" : ("; expires=" + expires.toGMTString())) + ((path === null) ? "" : ("; path=" + path)) + ((domain === null) ? "" : ("; domain=" + domain)) + ((secure === true) ? "; secure" : "");
    },

    /**
     * Retrieves cookies that are accessible by the current page. If a cookie
     * does not exist, <code>get()</code> returns <tt>null</tt>.  The following
     * example retrieves the cookie called "valid" and stores the String value
     * in the variable <tt>validStatus</tt>.
     * <pre><code>
     * var validStatus = Ext.util.Cookies.get("valid");
     * </code></pre>
     * @param {String} name The name of the cookie to get
     * @return {Object} Returns the cookie value for the specified name;
     * null if the cookie name does not exist.
     */
    getCookie : function(name){
        var arg = name + "=",
            alen = arg.length,
            clen = document.cookie.length,
            i = 0,
            j = 0;

        while(i < clen){
            j = i + alen;
            if(document.cookie.substring(i, j) == arg){
                return this.getCookieVal(j);
            }
            i = document.cookie.indexOf(" ", i) + 1;
            if(i === 0){
                break;
            }
        }
        return null;
    },

    /**
     * Removes a cookie with the provided name from the browser
     * if found by setting its expiration date to sometime in the past.
     * @param {String} name The name of the cookie to remove
     * @param {String} path (optional) The path for the cookie. This must be included if you included a path while setting the cookie.
     */
    clearCookie : function(name, path){
        if(this.getCookie(name)){
            path = path || '/';
            document.cookie = name + '=' + '; expires=Thu, 01-Jan-70 00:00:01 GMT; path=' + path;
        }
    },

    /**
     * @private
     */
    getCookieVal : function(offset){
        var endstr = document.cookie.indexOf(";", offset);
        if(endstr == -1){
            endstr = document.cookie.length;
        }
        return decodeURIComponent(document.cookie.substring(offset, endstr));
    },

    /**
     * JSON
     * von Mootools Json
     * @private
     */
    specialChars: {'\b': '\\b', '\t': '\\t', '\n': '\\n', '\f': '\\f', '\r': '\\r', '"' : '\\"', '\\': '\\\\'},

    /**
     * @private
     * @param {String} chr
     */
   	replaceChars: function(chr){
   		return Tx.specialChars[chr] || '\\u00' + Math.floor(chr.charCodeAt() / 16).toString(16) + (chr.charCodeAt() % 16).toString(16);
   	},
    /**
     * Wandelt Javascriptobject in String
     * @param obj
     * @return {String}
     */
   	encodeJson: function(obj){
        if (obj === null){
            return 'null';
        }
   		switch (Tx.typeOf(obj)){
   			case 'string':
   				return '"' + obj.replace(/[\x00-\x1f\\"]/g, Tx.replaceChars) + '"';
   			case 'array':
                var arr = [];
                for (var i=0, l=obj.length; i<l; i++){
                    arr.push(Tx.encodeJson(obj[i]));
                }
                return '['+arr.join(',')+']';
            case 'date':
                return Date.txFormat(obj, '"Y-m-d H:i:s"');
   			case 'object':
   				var string = [], key, value, json;
   				for (key in obj) {
                    value = obj[key];
   					json = Tx.encodeJson(value);
   					if (json) string.push(Tx.encodeJson(key) + ':' + json);
   				}
   				return '{' + string.join(',') + '}';
   			case 'number': case 'boolean': return String(obj);
   			case 'null': return 'null';
   		}
   		return null;
   	},
    /**
     * Wandelt String in Javascriptobject
     * @param string
     * @param secure
     */
   	decodeJson: function(string, secure){
   		if (Tx.typeOf(string) != 'string' || !string.length) return null;
   		if (secure && !(/^[,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]*$/).test(string.replace(/\\./g, '@').replace(/"[^"\\\n\r]*"/g, ''))) return null;
   		return eval('(' + string + ')');
   	},
    empty: function(){}

};/**
 * User Agents
 * Browser
 * -Opera 9: Opera/9.64 (Windows NT 6.1; U; de) Presto/2.1.1
 *
 * Boxes
 * -SMART: Mozilla+PVR; Inverto; Volksbox; 1.0; 1.0) CE-HTML/1.0
 * -TechniSat: Opera/10.60 (Linux sh4 ; U; HbbTV/1.1.1 (;;;;;); CE-HTML; TechniSat DIGIT ISIO S; en) Presto/2.6.33 Version/10.60
 * -VideoWeb: HbbTV/1.1.1 (;;;;) Mozilla/5.0 (compatible; ANTGalio/3.0.2.1.22.43.08; Linux2.6.18-7.1/7405d0-smp)
 * Samsung: Mozilla/5.0 (Windows; U; en-US; rv:1.8.1.11; Gecko/20071129; Firefox/2.5.0) Maple 5.1.00274 Navi
 * - Eviado: Opera/10.60 (Linux sh4 ; U; CE-HTML/1.0 (); de) Presto/2.6.33 Version/10.60  -> EVIADO_STB_2010 <- sollte eigentlich auch drin stehen
 *
 * TV Sets:
 * Samsung: Mozilla/5.0 (Windows; U; en-US; rv:1.8.1.11; Gecko/20071129; Firefox/2.5.0) Maple 5.1.00277 Navi
 * -Philips2010: Opera/9.70 (Linux mips ; U; CE-HTML/1.0 (<profilelist><ui_profile name=\"PHILIPS_OLS_2010\"/></profilelist>); en) Presto/2.2.1
 * -Philips2009: Opera/9.70 (Linux mips ; U; CE-HTML/1.0 (<profilelist><ui_profile name=\"PHILIPS_OLS_2008\"/></profilelist>); en) Presto/2.2.1
 * -Philips2011: Opera/9.70 (Linux mips; U; en) Trident/1.0 CE-HTML/1.0 NETTV/2.0.1
 *               Opera/9.70 (Linux mips; U; en) Trident/1.0 CE-HTML/1.0 NETTV/3.0.1
 *               Opera/9.70 (Linux mips; U; en) Mstar/1.0 CE-HTML/1.0 NETTV/1.0.1
 * -LG: Mozilla/5.0 (DirectFB; U; Linux mips; en) AppleWebKit/531.2+ (KHTML, like Gecko) Safari/531.2+ LG Browser/4.0.10(+mouse+SCREEN+TUNER; LGE; 32LD650P-NA; 08.01.00; 0x00000001;); LG NetCast.TV-2010
 * -LOEWE: Opera/9.80 (Linux sh4 ; U; HBBTV/1.0 (; LOH/1.00; -----;;;) CE-HTML/1.0 Config(L:de,CC:GB); en) Presto/2.5.21 Version/10.30
 * -Sharp: ---
 * -IKEA- Opera/9.80 (Linux armv6l; U; NETRANGEMMH;HbbTV/1.1.1;CE-HTML/1.0;IKEA LF1V350; en) Presto/2.8.115 Version/11.10
 *
 * Devicefilter über Subdomain setzen
 * smart.SERVICEDOMAIN
 * technisat.SERVICEDOMAIN
 * videoweb.SERVICEDOMAIN
 * eviado.SERVICEDOMAIN
 * samsung.SERVICEDOMAIN
 * philips.SERVICEDOMAIN
 * lg.SERVICEDOMAIN
 * loewe.SERVICEDOMAIN
 * sharp.SERVICEDOMAIN
 *
 * @namespace
 */

Tx.Device = {
    initialized : false,

    platform : false,
    version : false,

    b_isCEHTML : false,
    b_isHBBTV : false,
    b_isBOX : false,
    b_isTVSET : false,

    b_isOpera : false,  // Opera Browser for Development

    b_isIkea: false,
    b_isPhilips : false,
    b_isNetcast : false,
    b_isSharp : false,
    b_isLoewe : false,
    b_isSamsung : false,
    b_isVideoweb : false,
    b_isTechnisat : false,
    b_mouse: false,
    b_keyboard:false,
    b_isSmart : false,
    b_isEviado : false,
    b_isSetone: false,
    b_isVestel: false,
    b_isKeenhigh: false,
    b_isHdplus: false,
    b_isPanasonic: false,
    b_isToshiba: false,
    allowedDevices : [ 'cehtml','panasonic','smart', 'technisat', 'videoweb', 'eviado', 'samsung', 'philips', 'lg', 'loewe', 'sharp','setone','vestel','konstantinopel','keenhigh','hdplus','test','oranje','ikea','billy','toshiba']
};

Tx.Device.init = function() {
    if (this.initialized) return;

    if (this.identifyDeviceDomain()) {
        // Platform erfolgreich anhand der Domain identifiziert,
        // identifiziere nun die Version
        switch (this.platform) {
            case 'billy':
            case 'ikea':
                this.b_isIkea = true;
                this.version = 1;
                this.b_isCEHTML = true;
                break;
            case 'setone':
                this.b_isSetone = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'vestel':
            case 'konstantinopel':
                this.b_isVestel = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'keenhigh':
                this.b_isKeenhigh = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'hdplus':
                this.b_isHdplus = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'panasonic':
                this.b_isPanasonic = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'toshiba':
                this.b_isToshiba = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'cehtml':
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'test':
                this.b_isSetone = true;
                this.version=1;
                this.b_isCEHTML = true;
                break;
            case 'smart':
                this.b_isSmart = true;
                this.b_isCEHTML = true;
                this.version = 1;
                break;

            case 'technisat':
                this.b_isTechnisat = true;
                this.b_isCEHTML = true;
                this.b_isHBBTV = true;
                this.version = 1;
                break;

            case 'videoweb':
                this.b_isVideoweb = true;
                this.b_isCEHTML = true;
                this.b_isHBBTV = true;
                this.version = 1;
                break;

            case 'eviado':
                this.b_isEviado = true;
                this.b_isCEHTML = true;
                this.version = 1;
                break;

            case 'samsung':
                this.b_isSamsung = true;
                if (navigator.userAgent.match(/Maple 5/i)){
                    this.version = '2010';
                }
                else if (navigator.userAgent.match(/Maple 6/i)){
                    this.version = '2011'
                }
                else if (navigator.userAgent.match(/Maple 2012/i)){
                    this.version = '2012'
                }
                break;

            case 'philips':
            case 'oranje':
                this.b_isPhilips = true;
                this.b_isCEHTML = true;
                this.b_keyboard = true;
                this.b_mouse = true;
                if (navigator.userAgent.indexOf('PHILIPS_OLS_2008') > -1) {
                    this.version = '2009';
                } else if (navigator.userAgent.indexOf('PHILIPS_OLS_2010') > -1) {
                    this.version = '2010';
                    this.b_mouse = true;
                } else if (navigator.userAgent.indexOf('NETTV') > -1) {
                    this.version = '2011';
                    this.b_mouse = true;
                }
                break;

            case 'lg':
                this.b_isNetcast = true;
                this.b_isCEHTML = true;
                this.b_mouse = true;
                this.version = this.extractVersion('LG Browser', 11, '(');
                if (navigator.userAgent.indexOf('LG NetCast.TV') > -1) {
                    this.isTVSET = true;
                } else if (navigator.userAgent.indexOf('LG NetCast.Media') > -1) {
                    this.b_isBOX = true;
                }
                break;

            case 'loewe':
                this.b_isLoewe = true;
                this.b_isCEHTML = true;
                this.b_isHBBTV = true;
                this.version = this.extractVersion('LOH', 4, ';');
                break;

            case 'sharp':
                this.b_isSharp = true;
                this.b_isCEHTML = true;
                this.version = 1;
                break;


        }

    } else {
        Tx.Device.identifyByUseragent();
    }
    var ua = navigator.userAgent;
    //  ua.indexOf('CUS') > -1
    if (ua.indexOf('TechnoTrend') > -1){
        this.b_keyboard = true;
    }

    //debugMessage('DeviceMapper init: ' + this.platform+" - "+this.version);

    this.initialized = true;
};

Tx.Device.identifyDeviceDomain = function() {
    var myurl = document.URL || window.location.href || document.location.href;

    // Samsung Spezialität file://localhost/
    if (myurl.substring(0, 17) == 'file://localhost/') {
        this.platform = 'samsung';
        this.b_isTVSET = true;
        return true;
    }
    myurl = myurl.toLowerCase().replace(/^http[s]?:\/\//,'');   //str_replace('http://', '', str_replace('https://', '', myurl.toLowerCase()));

    var myplatform =  myurl.split('.')[0];

    if (this.allowedDevices.indexOf(myplatform) !== -1) {
        this.platform = myplatform;
        if (this.platform == 'oranje') this.platform = 'philips';
        if (this.platform == 'billy') this.platform = 'ikea';
        return true;
    }

    return false;
};

Tx.Device.identifyByUseragent = function() {
    // Platform konnte nicht anhand der Domain identifiziert werden
    var ua = navigator.userAgent.toLowerCase();
    if (ua.indexOf('opera') > -1 && ua.indexOf('ce-html') == -1) {
        // Opera Browser
        this.platform = 'opera';
        this.version = '9';
        this.b_isCEHTML = true;
        this.b_isOpera = true;
    }
    else if (ua.indexOf('firefox') > -1 && ua.indexOf('maple') == -1) {
        this.platform = 'firefox';
        this.version = 4;
        this.b_isFirefox = true;
        this.b_isCEHTML = false;
    }
    else if (ua.indexOf('ce-html') > -1 || ua.indexOf('philips') > -1 || ua.indexOf('nettv') > -1) {
        //CE HTML Devices
        this.b_isCEHTML = true;
        if (ua.indexOf('ikea') > -1){
            this.platform = 'ikea';
            this.version=1;
            this.b_isIkea = true;
            this.b_isTVSET = true;
        }
        else if (ua.indexOf('toshiba') > -1){
            this.platform = 'toshiba';
            this.version = 1;
            this.b_isToshiba = true;
            this.b_isCEHTML = true;
        }
        else if (ua.indexOf('philips_ols_2008') > -1) {
            // Philips 2009
            this.platform = 'philips';
            this.version = '2009';
            this.b_isPhilips = true;
            this.b_isTVSET = true;
            this.b_mouse = true;
            this.b_keyboard = true;
        } else if (ua.indexOf('philips_ols_2010') > -1) {
            // Philips 2009
            this.platform = 'philips';
            this.version = '2010';
            this.b_isPhilips = true;
            this.b_isTVSET = true;
            this.b_mouse = true;
            this.b_keyboard = true;
        } else if (ua.indexOf('nettv') > -1) {
            // Philips 2009
            this.platform = 'philips';
            this.version = '2011';
            this.b_isPhilips = true;
            this.b_mouse = true;
            this.b_isTVSET = true;
            this.b_keyboard = true;
        } else if (ua.indexOf('loh') > -1) {
            if (ua.indexOf('netfront') == -1){
                // Loewe
                this.platform = 'loewe';
                this.version = this.extractVersion('LOH', 4, ';');
                this.b_isHBBTV = true;
                this.b_isLoewe = true;
                this.b_isTVSET = true;
                this.b_keyboard = true;
            }
            else {
                this.platform = 'vestel';
                this.b_isVestel = true;
                this.b_isTVSET = true;
            }
        } else if (ua.indexOf('technisat') > -1) {
            // netcast
            this.platform = 'technisat';
            this.version = 1;
            this.b_isHBBTV = true;
            this.b_isTechnisat = true;
            this.b_isBOX = true;
        } else if (ua.indexOf('volksbox') > -1) {
            // netcast
            this.platform = 'smart';
            this.version = 1;
            this.b_isSmart = true;
            this.b_isBOX = true;
        }else if (ua.indexOf('technotrend') > -1){
            this.platform = 'technotrend';
            this.version = 1;
            this.b_isVideoweb = true;
            this.b_isHBBTV = true;
            this.b_isCEHTML = true;
            this.b_isBOX = true;
        }
    } else if (ua.search("lg browser") > -1 || ua.search("lg netcast.tv") > -1 || ua.search("lg netcast.media") > -1) {
        // netcast
        this.platform = 'netcast';
        this.version = this.extractVersion('LG Browser', 11, '(');
        this.b_isNetcast = true;
        this.b_isCEHTML = true;
        this.b_mouse = true;
        this.b_isBOX = ua.search("lg netcast.media") > -1;
    } else if (ua.indexOf('antgalio') > -1) {
        // Loewe
        this.platform = 'videoweb';
        this.version = 1;
        this.b_isVideoweb = true;
        this.b_isHBBTV = true;
        this.b_isCEHTML = true;
        this.b_isBOX = true;
    } else if (ua.indexOf('maple') > -1 && ua.indexOf('mozilla') > -1) {
        this.platform = 'samsung';
        this.b_isSamsung = true;
        if (ua.match(/maple 5/i)){
            this.version = '2010';
        }
        else if (ua.match(/maple 6/i)){
            this.version = '2011'
        }
    } else if (ua.indexOf('maple') > -1 && ua.indexOf('applewebkit') > -1) {
        this.platform = 'samsung';
        this.b_isSamsung = true;
        if (ua.match(/maple 2012/i)){
            this.version = '2012'
        }
        else {
            this.version = '2013';
        }
    } else if(ua.indexOf('applewebkit') > -1 && ua.indexOf('windows') > -1) {
        this.platform = 'windows';
        this.b_mouse = true;
        this.b_isCEHTML = true;
    } else {
        // Default use platform cehtml
        this.platform = 'cehtml';
        this.b_isCEHTML = true;
    }
};

Tx.Device.setPlatform = function(platform) {
    if (this.allowedDevices.indexOf(platform) !== -1) {
        this.platform = platform;
    }
};

Tx.Device.extractVersion = function(pattern, count, split) {
    var userAgent = navigator.userAgent;
    var re = new RegExp(pattern);
    var BrowserVer = userAgent.substring(userAgent.search(re) + count);
    if (split == null) {
        return BrowserVer;
    } else {
        return BrowserVer.split(split, 1);
    }
};

Tx.Device.getPlatform = function() {
    this.init();
    return this.platform;
};

Tx.Device.getVersion = function() {
    this.init();
    return this.version;
};

Tx.Device.get = function() {
    this.init();
    return this.platform + " " + this.version;
};

Tx.Device.isLowResolution = function() {
    return window.innerWidth > 900 && window.innerWidth < 1000;
};
/**
 * Check for CEHTML
 */
Tx.Device.isCEHTML = function() {
    this.init();
    return this.b_isCEHTML;
};

/**
 * Check for HBBTV
 */
Tx.Device.isHBBTV = function() {
    this.init();
    return this.b_isHBBTV;
};

/**
 * Check for TV-Set
 */
Tx.Device.isTVSET = function() {
    this.init();
    return this.b_isTVSET;
};

/**
 * Check for BOX
 */
Tx.Device.isBOX = function() {
    this.init();
    return this.b_isBOX;
};

/**
 * Check for Opera Browser
 */
Tx.Device.isOpera = function() {
    this.init();
    return this.b_isOpera;
};


/**
 * Check for Firefox Browser
 */
Tx.Device.isFirefox = function() {
    this.init();
    return this.b_isFirefox;
};
Tx.Device.isVestel = function() {
    this.init();
    return this.b_isVestel;
};
Tx.Device.isKeenhigh = function() {
    this.init();
    return this.b_isKeenhigh;
};
Tx.Device.isIkea = function() {
    this.init();
    return this.b_isIkea;
};
/**
 * Check for Philips TV Set
 */
Tx.Device.isPhilips = function() {
    this.init();
    return this.b_isPhilips;
};

/**
 * Check for LG TV Set
 */
Tx.Device.isNetcast = function() {
    this.init();
    return this.b_isNetcast;
};

/**
 * Check for Sharp TV Set
 */
Tx.Device.isSharp = function() {
    this.init();
    return this.b_isSharp;
};

/**
 * Check for Loewe TV Set
 */
Tx.Device.isLoewe = function() {
    this.init();
    return this.b_isLoewe;
};

/**
 * Check for Samsung Device
 */
Tx.Device.isSamsung = function() {
    this.init();
    return this.b_isSamsung;
};

/**
 * Check for VideoWeb Box
 */
Tx.Device.isVideoweb = function() {
    this.init();
    return this.b_isVideoweb;
};

/**
 * Check for Technisat Receiver
 */
Tx.Device.isTechnisat = function() {
    this.init();
    return this.b_isTechnisat;
};

Tx.Device.isToshiba = function() {
    this.init();
    return this.b_isToshiba;
};
/**
 * Check for Smart Box
 */
Tx.Device.isSmart = function() {
    this.init();
    return this.b_isSmart;
};

/**
 * Check for Eviado Box
 */
Tx.Device.isEviado = function() {
    this.init();
    return this.b_isEviado;
};
Tx.Device.hasMouse = function() {
    this.init();
    return !!(Tx.Device.isOpera() || (Tx.Device.isPhilips() && this.b_mouse === true) || Tx.Device.isNetcast() || Tx.Device.isFirefox() || this.platform == 'windows');

};
Tx.Device.hasKeyboard = function() {
    this.init();
    return this.b_keyboard;
};


/**
 * @author kasi
 */

Tx.applyIf(String.prototype, {
    /**
     * @lends String.prototype
     */
    trim: function() {
        return this.replace(/^\s+|\s+$/g, '');
    },
    pad: function(length, chr) {
        var s = this, len = Math.abs(length), count = len - s.length, tmp = [];
        chr = chr || '0';
        if (count <= 0) {
            return s;
        }
        while (count) {
            tmp[tmp.length] = chr;
            count --;
        }
        if (length > 0) {
            return tmp.join('') + s;
        }
        else {
            return s + tmp.join('');
        }
    },
    contains: function(string, separator) {
        return (separator) ? (separator + this + separator).indexOf(separator + string + separator) > -1 : this.indexOf(string) > -1;
    },
    clean: function() {
        return this.replace(/\s+/g, ' ').trim();
    },
    cut: function(len, text) {
        var s = this;
        text = text || ' ...';
        return (s.length <= len) ? s : s.substr(0, len - text.length) + text;
    }
});

Tx.applyIf(Array.prototype, {
    /**
     * @lends Array.prototype
     */
    indexOf: function(item, from) {
        var len = this.length;
        for (var i = (from < 0) ? Math.max(0, len + from) : from || 0; i < len; i++) {
            if (this[i] === item) return i;
        }
        return -1;
    },
    erase: function(item) {
        for (var i = this.length; i--;) {
            if (this[i] === item) this.splice(i, 1);
        }
        return this;
    },
    txType: 'array'
});
Tx.applyIf(Function.prototype, {
    /**
     * @lends Function.prototype
     */
    bind: function(bind) {
        var self = this, slice = Array.prototype.slice, args = slice.call(arguments, 1);
        return function() {
            return self.apply(bind, args.concat(slice.call(arguments)));
        };
    },
    delay: function(delay, bind, arg1, arg2, arg3, agr5, arg4, arg6, arg7, arg8, arg9) {
        return setTimeout(this.bind(bind, arg1, arg2, arg3, agr5, arg4, arg6, arg7, arg8, arg9), delay);
    },
    defer: function(bind, arg1, arg2, arg3, agr5, arg4, arg6, arg7, arg8, arg9){
        return setTimeout(this.bind(bind, arg1, arg2, arg3, agr5, arg4, arg6, arg7, arg8, arg9), 1);
    }

});
/*
 Format  Description                                                               Example returned values
 ------  -----------------------------------------------------------------------   -----------------------
 d     Day of the month, 2 digits with leading zeros                             01 to 31
 D     A short textual representation of the day of the week                     Mon to Sun
 j     Day of the month without leading zeros                                    1 to 31
 l     A full textual representation of the day of the week                      Sunday to Saturday
 F     A full textual representation of a month, such as January or March        January to December
 m     Numeric representation of a month, with leading zeros                     01 to 12
 M     A short textual representation of a month                                 Jan to Dec
 n     Numeric representation of a month, without leading zeros                  1 to 12
 Y     A full numeric representation of a year, 4 digits                         Examples: 1999 or 2003
 y     A two digit representation of a year                                      Examples: 99 or 03
 G     24-hour format of an hour without leading zeros                           0 to 23
 H     24-hour format of an hour with leading zeros                              00 to 23
 i     Minutes, with leading zeros                                               00 to 59
 s     Seconds, with leading zeros                                               00 to 59
 */
/**#@+
 * @static
 * @constant
 */
Date.deDayShort = ['So','Mo','Di','Mi','Do','Fr','Sa'];
Date.deDayLong = ['Sonntag','Montag','Dienstag','Mittwoch','Donnerstag','Freitag','Samstag'];
Date.deMonthShort = ['Jan','Feb', 'Mrz', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez'];
Date.deMonthLong = ['Januar','Februar', 'März', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember'];
Date.deDefaultFormat = 'd.m.Y H:i:s';
Date.txDayMilliseconds = 1000 * 60 * 60 * 24;
Date.TX_MILLI = 'milli';
Date.TX_SECOND = 'second';
Date.TX_MINUTE = 'minute';
Date.TX_HOUR = 'hour';
Date.TX_DAY = 'day';
Date.TX_MONTH = 'month';
Date.TX_YEAR = 'year';
/**#@-*/

/**
 * Wandelt ein Date-Objekt in einen formatierten String
 * <h4>Formatcodes</h4>
 * <pre><code>
 Format  Description                                                               Example returned values
 ------  -----------------------------------------------------------------------   -----------------------
 d     Day of the month, 2 digits with leading zeros                             01 to 31
 D     A short textual representation of the day of the week                     Mon to Sun
 j     Day of the month without leading zeros                                    1 to 31
 l     A full textual representation of the day of the week                      Sunday to Saturday
 F     A full textual representation of a month, such as January or March        January to December
 m     Numeric representation of a month, with leading zeros                     01 to 12
 M     A short textual representation of a month                                 Jan to Dec
 n     Numeric representation of a month, without leading zeros                  1 to 12
 Y     A full numeric representation of a year, 4 digits                         Examples: 1999 or 2003
 y     A two digit representation of a year                                      Examples: 99 or 03
 G     24-hour format of an hour without leading zeros                           0 to 23
 H     24-hour format of an hour with leading zeros                              00 to 23
 i     Minutes, with leading zeros                                               00 to 59
 s     Seconds, with leading zeros                                               00 to 59
 * </code></pre>
 * @static
 * @param {Date} dateObj
 * @param {String} format
 * @param {String} [lang=de]
 */
Date.txFormat = function(dateObj, format, lang) {
    if (Tx.typeOf(dateObj) != 'date') {
        return String(dateObj);
    }
    lang = lang || Tx.lang || 'de';
    format = format || Date[lang + 'DefaultFormat'] || Data.deDefaultFormat;
    var fullYear = dateObj.getFullYear(),
        year = String(fullYear).substr(2, 2),
        day = dateObj.getDay(),
        date = dateObj.getDate(),
        month = dateObj.getMonth(),
        hours = dateObj.getHours(),
        minutes = dateObj.getMinutes(),
        seconds = dateObj.getSeconds(),
        daysShort = Date[lang + 'DayShort'] || Data.deDayShort,
        daysLong = Date[lang + 'DayLong'] || Data.deDayLong,
        monthsShort = Date[lang + 'MonthShort'] || Data.deMonthShort,
        monthsLong = Date[lang + 'MonthLong'] || Data.deMonthLong,
        result = [];
    for (var chr, i = 0, l = format.length; i < l; i++) {
        chr = format.charAt(i);
        switch (chr) {
            case 'd':
                result.push(String(date).pad(2));
                break;
            case 'D':
                result.push(daysShort[day]);
                break;
            case 'j':
                result.push(date);
                break;
            case 'l':
                result.push(daysLong[day]);
                break;
            case 'F':
                result.push(monthsLong[month]);
                break;
            case 'm':
                result.push(String(month + 1).pad(2));
                break;
            case 'M':
                result.push(monthsShort[month]);
                break;
            case 'n':
                result.push(month + 1);
                break;
            case 'Y':
                result.push(fullYear);
                break;
            case 'y':
                result.push(year);
                break;
            case 'G':
                result.push(hours);
                break;
            case 'H':
                result.push(String(hours).pad(2));
                break;
            case 'i':
                result.push(minutes);
                break;
            case 's':
                result.push(String(seconds).pad(2));
                break;
            case '\\':
            case '%':
                result.push(format.charAt(i + 1));
                i++;
                break;
            default:
                result.push(chr);
        }
    }
    return result.join('');

};
/**
 * Wandelt einen String in ein Date-Objekt
 * @see Date.txFormat für Formatcodes
 * @static
 * @param {String} value
 * @param {String} format
 * @param {String} [lang=de]
 * @returns {Date|null}
 */
Date.txParse = function(value, format, lang) {
    if (Tx.typeOf(format) == 'array') {
        for (var idx = 0, len = format.length; idx < len; idx++) {
            var resDate = Date.txParse(value, format[idx], lang);
            if (resDate) {
                return resDate;
            }
        }
    }
    lang = lang || Tx.lang || 'de';
    format = format || Date[lang + 'DefaultFormat'] || Data.deDefaultFormat;
    value = String(value);
    var date = month = 1, year = new Date().getFullYear(),hour = minute = second = 0, chr, i, l, pos = 0;
    var monthsShort = Date[lang + 'MonthShort'] || Data.deMonthShort,
        monthsLong = Date[lang + 'MonthLong'] || Data.deMonthLong;
    var getIndexOf = function(arr, str) {
        str = str.toLowerCase();
        for (var i = 0, l = arr.length; i < l; i++) {
            if (str.indexOf(arr[i].toLowerCase()) === 0) {
                return i;
            }
        }
        return NaN;
    };
    for (i = 0,l = format.length; i < l; i++) {
        chr = format.charAt(i);
        switch (chr) {
            case 'd':
                date = parseInt(value.substr(pos, 2));
                if (isNaN(date)) return null;
                pos += 2;
                break;
            case 'j':
                date = parseInt(value.substr(pos, 2));
                if (isNaN(date)) return null;
                pos += String(date).length;
                break;
            case 'm':
                month = parseInt(value.substr(pos, 2));
                if (isNaN(month)) return null;
                pos += 2;
                break;
            case 'n':
                month = parseInt(value.substr(pos, 2));
                if (isNaN(month)) return null;
                pos += String(month).length;
                break;
            case 'y':
                year = parseInt(value.substr(pos, 2));
                if (isNaN(year)) return null;
                pos += String(year).length;
                break;
            case 'Y':
                year = parseInt(value.substr(pos, 4));
                if (isNaN(year)) return null;
                pos += 4;
                break;
            case 'H':
                hour = parseInt(value.substr(pos, 2));
                if (isNaN(hour)) return null;
                pos += 2;
                break;
            case 'G':
                hour = parseInt(value.substr(pos, 2));
                if (isNaN(hour)) return null;
                pos += String(hour).length;
                break;
            case 'i':
                minute = parseInt(value.substr(pos, 2));
                if (isNaN(minute)) return null;
                pos += 2;
                break;
            case 's':
                second = parseInt(value.substr(pos, 2));
                if (isNaN(second)) return null;
                pos += 2;
                break;
            case '\\':
            case '%':
                if (value.charAt(pos) != chr) return null;
                i++;
                pos++;
                break;
            case 'M':
                month = getIndexOf(monthsShort, value.substr(pos));
                if (isNaN(month)) return null;
                pos += monthsShort[month].length;
                month++;
                break;
            case 'F':
                month = getIndexOf(monthsLong, value.substr(pos));
                if (isNaN(month)) return null;
                pos += monthsLong[month].length;
                month++;
                break;
            default:
                if (value.charAt(pos) != chr) return null;
                pos++;
                break;
        }
    }
    return new Date(year, month - 1, date, hour, minute, second);
};
/**
 * @static
 * @param date
 * @param interval
 * @param value
 */
Date.txAdd = function(date, interval, value) {
    var d = new Date(date.getTime());
    if (!interval || value === 0) return d;

    switch (interval.toLowerCase()) {
        case Date.TX_MILLI:
            d.setMilliseconds(d.getMilliseconds() + value);
            break;
        case Date.TX_SECOND:
            d.setSeconds(d.getSeconds() + value);
            break;
        case Date.TX_MINUTE:
            d.setMinutes(d.getMinutes() + value);
            break;
        case Date.TX_HOUR:
            d.setHours(d.getHours() + value);
            break;
        case Date.TX_DAY:
            d.setDate(d.getDate() + value);
            break;
        case Date.TX_MONTH:
            /*
            var day = date.getDate();
            if (day > 28) {
                day = Math.min(day, Ext.Date.getLastDateOfMonth(Ext.Date.add(Ext.Date.getFirstDateOfMonth(date), 'mo', value)).getDate());
            }
            d.setDate(day);
            */
            d.setMonth(date.getMonth() + value);
            break;
        case Date.TX_YEAR:
            d.setFullYear(date.getFullYear() + value);
            break;
    }
    return d;
};
/**
 * @static
 * @param date
 */
Date.isLeapYear = function(date) {
    var year = Tx.typeOf(date) == 'date' ? date.getFullYear() : date;
    return !!((year & 3) == 0 && (year % 100 || (year % 400 == 0 && year)));
};

Tx.applyIf(Date.prototype, {
    /**
     * @lends Date.prototype
     */
    txType: 'date',
    /**
     * @see Date.isLeapYear
     */
    isLeapYear: function() {
        return Date.isLeapYear(this)
    },
    /**
     * @see Date.txFormat
     * @param format
     * @param lang
     */
    txFormat: function(format, lang) {
        return Date.txFormat(this, format, lang)
    },
    /**
     * @see Date.txParse
     * @param interval
     * @param value
     */
    txAdd: function(interval, value){
        return Date.txAdd(this, interval, value);
    }
});
/**
 * Element Klasse
 * @author kasi
 * @class
 */
TX_ELEMENT_DATA = {};
Tx.Element = function(id, initState){
    this.dom = id && id.tagName ? id : document.getElementById(id);
    if (initState){
        this.initState();
    }
};
Tx.Element.prototype = {
    /**
     * Überprüft, ob die Klasse className vorhanden ist
     * @param {string} className
     * @returns {boolean}
     */
    hasClass: function(className){
        return this.dom.className.contains(className, ' ');
    },
    /**
     * Fügt die Klasse className dem Element hinzu, wenn noch nicht vorhanden
     * @param {string} className
     * @returns {Tx.Element} this
     */
    addClass: function(className){
        var el = this.dom;
        if (!this.hasClass(className)) el.className = (el.className + ' ' + className).clean();
        return this;
    },
    /**
     * Entfernt die Klase className
     * @param {string} className
     * @returns {Tx.Element} this
     */
    removeClass: function(className){
        var el = this.dom;
        el.className = (el.className.replace(new RegExp('(^|\\s)' + className + '(?:\\s|$)'), '$1')).trim();
        return this;
    },
    /**
     * Wenn vorhanden wird die Klasse entfernt ansonsten hinzugefügt
     *
     * @param {string} className
     * @return {Tx.Element} this
     */
    toggleClass: function(className){
        return this[this.hasClass(className) ? 'removeClass' : 'addClass'](className);
    },
    /**
     * Initialisiert den Status (hasFocus und selected)
     */
    initState: function() {
        this.selected = this.hasClass('selected');
        this.hasFocus = this.hasClass('focus');
    },
    /**
     * Setzt /entfernt die Selected Klasse
     * @param state
     */
    setSelected: function(state){
        if (typeof state != 'undefined'){
            this.selected = !!state;
        }
        this[this.selected ? 'addClass' : 'removeClass']('selected');
    },
    /**
     * Setzt / entfernt die Focusklasse
     * @param state
     */
    setFocus: function(state){
        if (typeof state != 'undefined'){
            this.hasFocus = !!state;
        }
        if (this.hasFocus){
            Tx.focus(this);
        }
        else {
            this.removeClass('focus');
        }
    },
    /**
     * Setzt innerHHTML des Elements
     * @param {string|Array} html
     */
    setHtml: function(html){
        var el = this.dom;
        el.innerHTML = html ? ( typeof html != 'string' && html.join ? html.join('') : html) : '';
        return this;
    },
    appendChild: function(element){
        var el = this.dom;
        el.appendChild(element.dom || element);
        return this;
    },
    getId: function() {
        if (!this.id){
        var el = this.dom, id = el.getAttribute('id');
            if (!id){
                id = 'tx-el-'+Tx.id();
                el.setAttribute('id', id);
            }
            this.id = id;
        }
        return this.id;
    },
    store: function(name, value){
        var id = this.getId();
        if (!TX_ELEMENT_DATA[id]){
            TX_ELEMENT_DATA[id] = {};
        }
        if (typeof value == 'undefined'){
            return TX_ELEMENT_DATA[id][name];
        }
        else {
            TX_ELEMENT_DATA[id][name] = value;
            return this;
        }
    },
    cleareStore: function() {
        var id = this.getId();
        if (TX_ELEMENT_DATA[id]){
            delete TX_ELEMENT_DATA[id];
        }
        return this;
    },
    destroy: function() {
        var el = this.dom;
        this.cleareStore();
        el.parentNode.removeChild(el);
        return this;
    },
    attr: function(key, value) {
        var el = this.dom;
        if (typeof value == 'undefined'){
            return el.getAttribute(key);
        }
        else {
            if (typeof value == 'function'){
                value = value(el, key);
            }
            el.setAttribute(key, value);
            return this
        }
    },
    /**
     * Fügt ein Element ein
     *
     * @param {HTMLElement|Tx.Element} element
     * @param {String} [where=bottom] Position top, bottom, before, after
     */
    insertChild: function(element, where){
        var el = this.dom, first = el.firstChild, parent = el.parentNode;
        element = element.dom || element;
        where = where || 'bottom';
        switch(where.toLowerCase()){
            case 'top': el.insertBefore(element, first); break;
            case 'bottom': el.appendChild(element); break;
            case 'before' : if (parent) parent.insertBefore(element, el); break;
            case 'after' : if (parent) parent.insertBefore(element, el.nextSibling); break;
        }
        return this;
    },
    insertHtml: function(html, where){
        var fragment = document.createDocumentFragment(), div = document.createElement('div');
        div.innerHTML = Tx.typeOf(html) == 'array' ? html.join('') : html;
        for (var i=0, l=div.childNodes.length; i<l; i++){
            fragment.appendChild(div.childNodes[i]);
        }
        this.insertChild(fragment, where);
    },
    /**
     * Liefert innerHTML des Elements zurück
     * @returns {string}
     */
    getHtml: function() {
        var el = this.dom;
        return el.innerHTML;
    },

    /**
     * Liefert den aktuellen Stil des Element für den Wert property
     * @param {string} property
     * @returns {string}
     */
    getCurrentStyle: function(property){
        var el = this.dom;
        if (el.currentStyle) return el.currentStyle[property];
        var defaultView = document.defaultView,
            computed = defaultView ? defaultView.getComputedStyle(el, null) : null;
        return (computed) ? computed.getPropertyValue(property) :el.style[property];
    },
    /**
     * Setzt den Stil des Elements
     * @param {string} property
     * @param {string} value
     * @returns {Tx.Element}
     */
    setStyle: function(property, value){
        var el = this.dom;
        el.style[property] = value;
        return this;
    },
    /**
     * Setzt mehrere Stile des Elements gleichzeitig
     * @param {object} values
     * @returns {Tx.Element}
     */
    setStyles: function(values){
        var k;
        for(k in values){
            this.setStyle(k, values[k]);
        }
        return this;
    },
    /**
     * Versteckt das Element
     * @returns {Tx.Element}
     */
    hide: function(){
        var el = this.dom;
        el.style.display = 'none';
        return this;
    },
    /**
     * Zeigt das Element
     * @param {string} [display=block]
     * @returns {Tx.Element}
     */
    show: function(display){
        var el = this.dom;
        el.style.display = display || 'block';
        return this;
    },
    /**
     * Kurz für el.dom.addEventListener(name, fn, false)
     * @param {string} name
     * @param {function} fn
     * @returns {Tx.Element}
     */
    addEventListener: function(name, fn){
        var el = this.dom;
        el.addEventListener(name, fn, false);
        return this;
    },
    /**
     * Kurz für el.dom.removeEventListener(name, fn, false)
     * @param {string} name
     * @param {function} fn
     * @returns {Tx.Element}
     */
    removeEventListener: function(name, fn){
        var el = this.dom;
        el.removeEventListener(name, fn, false);
        return this;
    },
    /**
     * Findet Unterelemente des elements mit einem einfachen Selector
     * @param {string} selector
     * @returns {Tx.Elements}
     */
    query: function(selector){
        return Tx.query(selector, this.dom);
    },
    /**
     * Scrollt den Inhalt des Element um  amount Pixel nach oben
     * @param {int} amount
     * @returns {Tx.Element}
     */
    scrollUp: function(amount){
        var el = this.dom;
        el.scrollTop = Math.max(0, el.scrollTop-amount);
        return this;
    },
    /**
     * Scrollt den Inhalt des Element um  amount Pixel nach unten
     * @param {int} amount
     * @returns {Tx.Element}
     */
    scrollDown: function(amount){
        var el = this.dom, maxScrollTop = Math.max(0, el.scrollHeight - el.clientHeight);
        el.scrollTop = Math.min(maxScrollTop, el.scrollTop + amount);
        return this;
    },
    /**
     * Scrollt den Inhalt des Element an die Stelle pos
     * @param {int} pos
     * @returns {Tx.Element}
     */
    scrollTo: function(pos){
        var el = this.dom, maxScrollTop = Math.max(0, el.scrollHeight - el.clientHeight);
        el.scrollTop = Math.max(0,Math.min(maxScrollTop, pos));
        return this;
    },
    /**
     * Liefer ein Array mit allen direkten Kindelements, die dem Selector entsprechen
     * Wenn noExtend true ist werden die Elemente nicht erweitert
     * @param {string} selector
     * @param {boolean} noExtend
     * @returns {HTMLElement[]|Tx.Element[]}
     */
    getChildren: function(selector, noExtend) {
        var tagName, className, result=[], temp, child;
        selector = selector || '*';
        temp = selector.split('.');
        tagName = temp[0].toUpperCase();
        className = temp[1] || false;
        for (var i=0, l=this.dom.childNodes.length; i<l; i++){
            child = this.dom.childNodes[i];
            if (child.nodeType === 1 && (tagName == '*' || child.tagName.toUpperCase() == tagName)){
                if (className){
                    if (child.className.indexOf(className) !== -1){
                        result.push(noExtend ? child : Tx.get(child));
                    }
                }
                else {
                    result.push(noExtend ? child : Tx.get(child));
                }
            }
        }
        return result;
    },
    /**
     * Führt einen Click event auf dem Element aus
     * @returns {Tx.Element}
     */
    click: function() {
        var el = this.dom;
        if(typeof el.click == 'function'){
            el.click();
        }
        else if (typeof el.onclick == 'function'){
            el.onclick();
        }
        else if (typeof el.onclick == 'string'){
            try {
                eval(el.onclick);
            }
            catch (e){}
        }
        return this;
    }

};
/**
 * @class
 */
Tx.Elements = function() {
    this.elements = [];
};
Tx.Elements.prototype = {
    /**
     * Fügt der Liste Elemente, die dem einfachen Selector entsprechen hinzu
     * @see Tx.query
     * @param {string} selector
     * @param {HTMLElement} [base=document]
     * @returns {Tx.Elements}
     */
    add: function(selector, base) {
        var els = this.elements;
        base = base || document;
        if (!selector){
            return this;
        }
        if (typeof selector == 'string'){
            if (window.Sizzle){
                Sizzle(selector, base, this.elements);
            }
            else {
                var parts = selector.split('.'), tagName = parts[0] || '*', className = parts[1], elements = base.getElementsByTagName(tagName), element;
                for (var i=0, l=elements.length; i<l; i++){
                    element = elements[i];
                    if (!className || Tx.fly(element).hasClass(className)){
                        els[els.length] = element;
                    }
                }
            }
        }
        else if (selector.dom) {
            els[els.length] = selector.dom;
        }
        else if (selector.tagName){
            els[els.length] = selector;
        }
        return this;
    },
    /**
     * Fügt der Liste ein element mit der Id id hinzu
     * @param {int} id
     * @returns {Tx.Elements}
     */
    addId: function(id){
        var els = this.elements, el = document.getElementById(id);
        if (el) {
            els[els.length] = el;
        }
        return this;
    },
    /**
     * Ruft die Funktion fn mit jedem Element (fn(element, index)).
     * Wenn fn ein String ist wird die entsprechende Methode von Tx.Element aufgerufen
     * @private
     * @param {function|string} fn
     * @param {object} [scope=this]
     */
    each: function(fn, scope){
        var slice = Array.prototype.slice, args = slice.call(arguments, 1), element, el;
        for (var i=0, l=this.elements.length; i<l; i++){
            element = this.elements[i];
            if (typeof fn == 'string'){
                el = Tx.fly(element);
                el[fn].apply(el, args);
            }
            else {
                fn.call(scope || this, element, i);
            }
        }
        return this;
    },
    filter: function(fn){
        var els = new Tx.Elements();
        if (typeof fn == 'string'){
            els.elements = Sizzle.matches(fn,this.elements);
        }
        else {
            for (var i=0, l=this.elements.length; i<l; i++){
                if (fn(this.elements[i],i)){
                    els.add(this.elements[i]);
                }
            }
        }
        return els;
    },
    /** Wrapper Funktionen */
    addClass: function(cls){this.each('addClass',cls)},
    removeClass: function(cls){this.each('removeClass',cls)},
    setStyle: function(name, value){this.each('setStyle',name, value)},
    setStyles: function(styles){this.each('setStyles',styles)},
    show: function(){this.each('show')},
    hide: function(){this.each('hide')},
    setFocus: function(state){
        if (state){
            var el = this.elements[0];
            if (el){
                Tx.get(el).setFocus(true);
            }
        }
        else {
            this.each('setFocus',state)
        }
    },
    setSelected: function(state){this.each('setSelected',state)},
    addEventListener: function(name, fn){this.each('addEventListener',name, fn)},
    setHtml: function(html){this.each('setHtml',html)},
    attr: function(name,value){
        var res = [];
        this.each(function(el){
            res.push(Tx.fly(el).attr(name, value));
        });
        return res;
    },
    get: function(id){
        var el = this.elements[id];
        if (el){
            return Tx.get(el);
        }
        return null;
    },
    fly: function(id){
        var el = this.elements[id];
        if (el){
            return Tx.fly(el);
        }
        return null;
    }
};
Tx._fly = new Tx.Element(Tx.getBody());
/**
 * Basisklasse für alle Tx Objekte
 * @author kasi
 * @class
 */

Tx.Base = Tx.extend(Object, {
    /**
     * @lends Tx.Base.prototype
     */
    _events: null,
    /**
     */
    constructor: function() {
        this._events = {};
    },
    /**
     * Fügt dem Objekt einen event hinzu
     *
     * @param {String} name
     * @param {Function}fn
     * @param {Object} [scope=this]
     * @param {Array} [args] Zusätzliche Argumente, die beim Aufruf von fn angehängt werden
     */
    addEvent: function(name, fn, scope, args){
        if (!this._events[name]){
            this._events[name] = [];
        }
        this._events[name].push({
            fn: fn,
            scope: scope || this,
            args : args || []
        });
    },
    /**
     * Fügt dem Objekt mehrere Events hinzu
     * @param {object} events
     */
    addEvents: function(events){
        var k, event, scope;
        for (k in events){
            event = events[k];
            if (k == 'scope'){
                scope = event;
                continue;
            }
            if (typeof event == 'function'){
                this.addEvent(k, event)
            }
            else {
                this.addEvent(k, event.fn, event.scope || scope, event.args)
            }
        }
    },
    /**
     * Entfernt einen Event vom Objekt
     * @param {string} name
     * @param {function} fn
     */
    removeEvent: function(name, fn){
        var entry;
        if (this._events[name]){
            for(var list = this._events[name], i=list.length; i--;){
                entry = list[i];
                if (entry.fn === fn){
                    list.splice(i, 1)
                }
            }
        }
    },
    /**
     * Entfernt alle Events vom Objekt
     */
    removeAllEvents: function() {
        var name, i, l, event, events = this._events;
        for(name in events){
            if (events.hasOwnProperty(name)){
                for (i=0, l = events[name].length; i<l; i++){
                    event = events[name][i];
                    if(event){
                        event.fn = null;
                        event.scope = null;
                        event.args = null;
                    }
                    events[name][i] = null;
                }
                events[name] = [];
            }
        }
        this._events = {};
    },
    /**
     * Feuert einen Event
     * @param {String} name
     * @param {Array} args
     */
    fireEvent: function(name, args){
        var entry;
        if (this._events[name]){
            for(var i=0, list=this._events[name], l=list.length; i<l; i++){
                entry = list[i];
                if (entry.fn.apply(entry.scope, [this].concat(args || []).concat(entry.args)) === false){
                    return false;
                }
            }
        }
    }
});/**
 * XMLHttp Request Klasse und jsonp Request
 * @author kasi
 * @class   504138
 */

Tx.Ajax = Tx.extend(Tx.Base, {
    /**
     * @lends Tx.Ajax.prototype
     */
    xhr: null,
    method: 'post',
    running: false,
    async: true,
    isJson: true,
    successStatus: 200,
    jsonpCallbackKeyname: 'callback',
    jsonpCallbackName: 'jsonpCallback',
    jsonpScriptId: 'jsonpscript',
    jsonpTimeout: 5000,
    encodeParamSeparator: '&',
    encoding: 'utf-8',
    /**
     * @param {object} cfg
     */
    constructor: function(cfg){
        Tx.Base.prototype.constructor.call(this);
        Tx.apply(this, cfg);
        this.boundOnStateChange = this.onStateChange.bind(this);
    },
    /**
     * Startet einen Request zum Webserver
     * <pre><code>
     *    var ajax = new Tx.Ajax();
     *    ajax.request({
     *        url:'http://service.webserver.de',
     *        success: function(request, data) {
     *             // mach was mit data
     *        }
     *    })
     * </code></pre>
     * @param {Object} options
     */
    request: function(options){
        if (Tx.debugging && TXConfig.debugRequests && this.debug !== true){
            Tx.info('REQUEST',options, this);
        }
        this.success = null;
        this.failure = null;
        Tx.apply(this, options);
        if (this.method == 'jsonp'){
            this.doJsonpRequest();
        }
        else {
            this.doRequest();
        }
    },
    /**
     * @private
     * @param params
     * @param b
     */
    encodeParams: function(params, b){
        var result = [];
        for (var k in params){
            result.push(encodeURIComponent(k)+'='+encodeURIComponent(params[k]));
        }
        return result.join(b || this.encodeParamSeparator)
    },
    /**
     * Stopt den Request
     */
    cancel: function(){
        this.running = false;
        if (this.xhr){
            this.xhr.abort();
            this.xhr.onreadystatechange = function(){};
            if (typeof this.xhr.destroy == 'function'){
                this.xhr.destroy();
            }
        }
        if (this.method != 'jsonp'){
            this.xhr = new XMLHttpRequest();
        }
        else {
            var script = Tx.byId(this.jsonpScriptId);
            if (script){
                Tx.getHead().removeChild(script);
            }
        }
    },
    /**
     * @private
     */
    doRequest: function() {
        var method = this.method.toUpperCase(),
            params = this.params || {},
            url = this.url,
            async = this.async;
        try {
            params._dc = new Date().getTime();
            params = this.encodeParams(params);
            this.cancel();
            this.xhr.onreadystatechange = this.boundOnStateChange;
            this.xhr.open(method,url, async);
            if (method == 'POST'){
                var encoding = (this.encoding) ? '; charset=' + this.encoding : '';
                try {
                    this.xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded' + encoding);
                    this.xhr.setRequestHeader('Content-length',params.length);
                }
                catch(e){}
            }
            this.fireEvent('start');
            this.running = true;
            this.xhr.send(params);
        }
        catch(e){
            this.message = String(e);
            this.handleFailure();
            return;
        }
        if (!this.async){
            this.onStateChange();
        }
    },
    /**
     * @private
     */
    doJsonpRequest: function() {
        var params = this.params || {},
            attribs = {},
            url = this.url;
        window[this.jsonpCallbackName] = this.jsonpCallback.bind(this);
        try {
            params._dc = new Date().getTime();
            params[this.jsonpCallbackKeyname] = this.jsonpCallbackName;
            params = this.encodeParams(params);
            this.cancel(true);
            attribs.src = url + '?' +params;
            attribs.id = this.jsonpScriptId;
            this.fireEvent('start');
            this.running = true;
            Tx.addScript(attribs);
            this.jsonpTimer = window.setTimeout(this.onJsonpTimeout.bind(this), this.jsonpTimeout);
        }
        catch(e){
            this.message = String(e);
            this.handleFailure();
        }
    },
    /**
     * @private
     */
    onJsonpTimeout: function() {
        this.message = 'timeout';
        this.handleFailure();
    },
    /**
     * @private
     * @param data
     */
    jsonpCallback:function(data){
        if (this.jsonpTimer){
            window.clearTimeout(this.jsonpTimer);
        }
        if (!data){
            this.message = 'no data';
            this.handleFailure();
            return;
        }
        if (typeof this.success == 'function'){
            this.success.call(this.scope || this, this, data);
        }
        this.fireEvent('finished', [data]);
    },
    /**
     * @private
     */
    handleSuccess: function() {
        var data = this.xhr.responseText;
        this.running = false;
        if (this.isJson){
            try {
                data = this.parseJson(data);
            }
            catch(e){
                this.message = 'invalid json';
                this.handleFailure();
                return;
            }
        }
        if (!data){
            this.message = 'no data';
            this.handleFailure();
            return;
        }
        if (typeof this.success == 'function'){
            this.success.call(this.scope || this, this, data);
        }
        this.fireEvent('finished', [data]);
        if (Tx.debugging && TXConfig.debugRequests && this.debug !== true){
            Tx.info('REQUEST','finished', data, this);
        }
    },
    /**
     * private
     */
    handleFailure: function() {
        if (this.jsonpTimer){
            window.clearTimeout(this.jsonpTimer);
        }
        this.running = false;
        this.fireEvent('finished');
        if (typeof this.failure == 'function'){
            this.failure.call(this.scope || this, this, this.message);
        }
        if (Tx.debugging && TXConfig.debugRequests && this.debug !== true){
            Tx.info('REQUEST','failure', this);
        }
    },
    /**
     * @private
     * @param data
     */
    parseJson: function(data) {
        return eval('('+data+')')
    },
    /**
     * @private
     */
    onStateChange: function(){
        try {
            this.state = this.xhr.readyState;
        }
        catch(e) {
            this.state = 0;
        }
        if (this.state == 4){
            if (this.xhr.status == this.successStatus){
                this.handleSuccess();
            }
            else {
                this.handleFailure();
            }
        }
    }
});

/**
 * @author kasi
 */
/**#@+
 @constant
 */

var VK_LEFT = VK_LEFT || 37,
    VK_RIGHT =VK_RIGHT || 39,
    VK_UP = VK_UP || 38,
    VK_DOWN = VK_DOWN || 40,
    VK_ENTER = VK_ENTER || 13,
    VK_0 = VK_0 || 48,
    VK_1 = VK_1 || 49,
    VK_2 = VK_2 || 50,
    VK_3 = VK_3 || 51,
    VK_4 = VK_4 || 52,
    VK_5 = VK_5 || 53,
    VK_6 = VK_6 || 54,
    VK_7 = VK_7 || 55,
    VK_8 = VK_8 || 56,
    VK_9 = VK_9 || 57,
    VK_RED = VK_RED || 82, //R
    VK_GREEN = VK_GREEN || 71, //G
    VK_YELLOW = VK_YELLOW || 89, //Y
    VK_BLUE = VK_BLUE || 66,  //B
    VK_PLAY = VK_PLAY || 179,
    VK_STOP = VK_STOP ||178,
    VK_PAUSE = VK_PAUSE || 19,
    VK_REWIND = VK_REWIND || 177,
    VK_FAST_FWD = VK_FAST_FWD || 176,
    VK_INFO = VK_INFO || 73, //I
    VK_BACK = VK_BACK || 219, //ß
    VK_BACK_SPACE = VK_BACK_SPACE || 219, //ß
    VK_EXIT = VK_EXIT || 88, //x
    VK_VOLUME_UP = VK_VOLUME_UP || 0,
    VK_VOLUME_DOWN = VK_VOLUME_DOWN || 0,
    VK_CHANNEL_UP = VK_CHANNEL_UP || 0,
    VK_CHANNEL_DOWN = VK_CHANNEL_DOWN || 0,
    VK_MUTE = VK_MUTE || 0;

var TX_KEY_UNDEFINED = 0,
    TX_KEY_LEFT = 1,
    TX_KEY_RIGHT = 2,
    TX_KEY_UP = 3,
    TX_KEY_DOWN = 4,
    TX_KEY_ENTER = 5,
    TX_KEY_0 = 6,
    TX_KEY_1 = 7,
    TX_KEY_2 = 8,
    TX_KEY_3 = 9,
    TX_KEY_4 = 10,
    TX_KEY_5 = 11,
    TX_KEY_6 = 12,
    TX_KEY_7 = 13,
    TX_KEY_8 = 14,
    TX_KEY_9 = 15,
    TX_KEY_RED = 16,
    TX_KEY_GREEN = 17,
    TX_KEY_YELLOW = 18,
    TX_KEY_BLUE = 19,
    TX_KEY_PLAY = 20,
    TX_KEY_STOP = 21,
    TX_KEY_PAUSE = 22,
    TX_KEY_REWIND = 23,
    TX_KEY_FORWARD = 24,
    TX_KEY_PREV = 25,
    TX_KEY_NEXT = 26,
    TX_KEY_INFO = 27,
    TX_KEY_EXIT = 28,
    TX_KEY_BACK = 29,
    TX_KEY_RETURN = 30;
    TX_KEY_PORTAL = 31;
    TX_KEY_VOLUME_UP = 32;
    TX_KEY_VOLUME_DOWN = 33;
    TX_KEY_CHANNEL_UP = 34;
    TX_KEY_CHANNEL_DOWN = 35;
    TX_KEY_MUTE = 36;
/**#@-*/

/**
 * @class
 */
Tx.Remote = Tx.extend(Tx.Base, {
    /**
     * @lends Tx.Remote.prototype
     */
    map: [],
    constructor: function() {
        Tx.Base.prototype.constructor.call(this);
    },
    /**
     * Liefert einen TX_KEY_???? code von einem Event zurück
     * @param {Event} event
     * @returns {int}
     */
    getKeyFromEvent: function(event) {
        var code = event.keyCode, key = this.map[code];
        if (typeof key == 'undefined' || key === 0 || key === null || code === 0) {
            return TX_KEY_UNDEFINED;
        }
        else {
            return key;
        }
    }
});/**
 * @author kasi
 */

/**#@+
 * @constant
 */
var TX_STATE_STOPPED = 0,
    TX_STATE_PLAYING = 1,
    TX_STATE_PAUSED = 2,
    TX_STATE_CONNECTING = 3,
    TX_STATE_BUFFERING = 4,
    TX_STATE_FINISHED = 5,
    TX_STATE_ERROR = 6,
    TX_VIDEO_NETWORK_ERROR = 1,
    TX_VIDEO_NOT_FOUND_ERROR = 2;
    TX_VIDEO_CONNECTION_FAILED = 3;
/**#@-*/

/**
 * @class
 */
Tx.Video = Tx.extend(Tx.Base, {
    /**
     * @lends Tx.Video
     */
    top:100,
    left:100,
    width:640,
    height:400,
    isFullscreen: false,
    state:TX_STATE_STOPPED,
    checkInterval: 1000,
    isLivestream: false,
    duration:0,
    constructor: function(config){
        Tx.Base.prototype.constructor.call(this);
        Tx.apply(this, config);
    },
    init: function(){},
    load: function(video){},
    play: function(){},
    playLivestream: function(url){},
    pause: function() {},
    stop: function() {},
    forward: function(seconds){},
    rewind: function(seconds){},
    destroy: function() {},
    startCheckTimer: function() {
        this.stopCheckTimer();
        this.checkTimer = setInterval(this.onCheckTimer.bind(this), this.checkInterval);
    },
    stopCheckTimer: function() {
        try {
            if (this.checkTimer){
                clearInterval(this.checkTimer);
            }
        }
        catch(e){}
    },
    onCheckTimer: function() {},
    setDimension: function() {},
    setFullscreen: function() {
        this.setDimension(0,0,window.innerWidth, window.innerHeight);
        this.isFullscreen = true;
    },
    setWindow: function() {
        this.setDimension(this.left, this.top, this.width, this.height);
        this.isFullscreen = false;
    },
    setDuration: function(duration) {
        this.duration = Number(duration);
    }
});
/**
 * @author kasi
 * @class
 * @static
 */
Tx.NavigationManager = new (Tx.extend(Tx.Base, {
    /**
     * @lends Tx.NavigationManager
     */
    navigations: {},
    currentNavigation: null,
    lastNavigation: null,
    activeItem: null,
    isFullscreen: false,
    viewClass: '',

    constructor: function() {
        Tx.Base.prototype.constructor.call(this);
    },
    registerNavigation: function(navigation){
        this.navigations[navigation.getId()] = navigation;
    },
    getNavigation: function(id) {
        return this.navigations[id];
    },
    selectNavigation: function(id){
        var nav = this.navigations[id];
        Tx.log('selectNavigation '+id+' '+(nav ? '' : ' not found'));
        if (nav){
            if (this.fireEvent('beforechange', [this.currentNavigation, nav]) === false){
                return false;
            }
            if (this.currentNavigation){
                this.lastNavigation = this.currentNavigation.getId();
                this.currentNavigation.deselect();
            }
            this.currentNavigation = nav;
            nav.select();
            this.fireEvent('changed', [nav]);
            return nav;
        }
    },
    deactivateAll: function() {
        var navigations = this.navigations, nav;
        for (var key in this.navigations){
            nav = this.navigations[key];
            if (nav && nav.activeItem > -1){
                nav.deactivate();
            }
        }
    },
    activateNavigationItem: function(navId, itemId){
        var nav;
        this.deactivateAll();
        nav = this.selectNavigation(navId);
        if (nav){
            nav.activate(itemId);
            this.fireEvent('activate',[nav]);
        }

    },
    selectLastNavigation: function() {
        if (this.lastNavigation){
            this.selectNavigation(this.lastNavigation);
            this.lastNavigation = null;
        }
    },
    getCurrentNavigation: function() {
        return this.currentNavigation;
    },
    handleKey: function(){

    }
}));

Tx.Navigation = Tx.extend(Tx.Base, {
    activeClass: 'active',
    focusClass: Tx.focusClass,
    disabledClass: 'disabled',
    activeItem: -1,
    focusItem: -1,
    orientation: 'horizontal',
    left: null,
    right: null,
    up: null,
    down: null,
    defaultItem: 0,
    enabled: true,

    constructor: function(config){
        Tx.Base.prototype.constructor.call(this);
        Tx.apply(this, config);
        if (this.events){
            this.addEvents(this.events);
            delete this.events;
        }
        if(this.container){
            this.container = Tx.get(this.container);
            this.id = this.container.dom.id;
        }
        Tx.NavigationManager.registerNavigation(this);
        this.items = [];
        if (!this.enabled){
            this.setEnabled(this.enabled);
        }
    },
    getId: function() {
        return this.id || (this.id = 'NAV'+Tx.id());
    },
    setEnabled: function(enabled){
        this.enabled = enabled !== false;
        this.container[this.enabled ? 'removeClass' : 'addClass'](this.disabledClass);
    },
    isEnabled: function() {
        return this.enabled;
    },
    activate: function(itemId) {
        this.deactivate();
        if (typeof itemId != 'undefined'){
            this.activeItem = itemId;
        }
        else {
            this.activeItem = this.focusItem;
        }
        var item = this.getItem(this.activeItem);
        if (item){
            item.addClass(this.activeClass);
            this.fireEvent('activate', [item]);
        }
        else {
            this.activeItem = -1;
        }
    },
    deactivate: function() {
        if (this.activeItem >= 0){
            var item = this.getItem(this.activeItem);
            if (item){
                item.removeClass(this.activeClass);
                this.fireEvent('deactivate', [item]);
            }
            this.activeItem = -1;
        }
    },
    deselect: function() {
        if (this.focusItem >= 0){
            var item = this.getItem(this.focusItem);
            if (item){
                Tx.blur();
            }
        }
    },
    select: function() {
        if (this.focusItem == -1){
            this.focusItem = this.defaultItem;
        }
        if (this.focusItem !== -1){
            var item = this.getItem(this.focusItem);
            if (item){
                Tx.focus(item.getContainer());
            }
        }
    },
    first: function() {
        this.deselect();
        this.focusItem = 0;
        this.select();
        return true;
    },
    last: function() {
        this.deselect();
        this.focusItem = this.items.length-1;
        this.select();
        return true;
    },
    next: function() {
        if (!this.isFocusLast()){
            this.deselect();
            this.focusItem++;
            this.select();
            return true;
        }
        return false;
    },
    prev: function() {
        if (this.focusItem > 0){
            this.deselect();
            this.focusItem--;
            this.select();
            return true;
        }
        return false;
    },
    isFocusLast: function() {
        return this.focusItem >= this.items.length -1;
    },
    isFocusFirst: function(){
        return this.focusItem === 0;
    },
    getContainer: function() {
        return this.container;
    },
    getItem: function(id){
        return this.items[id];
    },
    getActiveItem: function() {
        return this.items[this.activeItem];
    },
    getFocusItem: function(){
        return this.items[this.focusItem];
    },
    getFocusItemId: function() {
        return this.focusItem > -1 ? this.items[this.focusItem].getId() : null;
    },
    addItem: function(config){
        var item;
        if (config instanceof Tx.NavigationItem){
            item = config;
        }
        else {
            item = new Tx.NavigationItem(this, config);
        }
        if (item.refName){
            this[item.refName] = item;
        }
        this.items.push(item);
    },
    addItems: function(items) {
        for(var i=0, l=items.length; i<l; i++){
            this.addItem(items[i]);
        }
    },
    handleItemClicked: function(item, event, container) {
        Tx.NavigationManager.selectNavigation(this.getId());
        this.fireEvent('itemclicked', [item, event, container]);
    }
});

Tx.NavigationItem = Tx.extend(Tx.Base, {
    navigation: null,
    left: null,
    right: null,
    up: null,
    down: null,
    constructor: function(navigation, config){
        Tx.Base.prototype.constructor.call(this);
        Tx.apply(this, config);
        this.navigation = navigation;
        if (this.events){
            this.addEvents(this.events);
            delete this.events;
        }
        if (this.html && this.html.push){
            this.html = this.html.join('');
        }
        if (this.container){
            this.container = Tx.get(this.container);
            //this.container.addEvent('click',this.handleClick.bind(this));
            this.id = this.container.dom.id;
        }
    },
    getId: function(){
        return this.id || (this.id = 'Item'+Tx.id());
    },
    handleClick: function(event){
        this.fireEvent('click', [event, this.container]);
        this.navigation.handleItemClicked(this, event, this.container);
    },
    getContainer: function(){
        return this.container;
    },
    addClass: function(className){
        this.container.addClass(className);
    },
    removeClass: function(className){
        this.container.removeClass(className);
    },
    setHtml: function(html){
        this.container.setHtml(html);
        return this;
    },
    getHtml: function() {
        return this.container.getHtml();
    }
});

/**
 * Config Object
 * @namespace
 */

TXConfig = {
    environment: 'development',
    debugging: false,
    debugRequests:false,
    videoCfg: {
        top:100,
        left:200,
        width:640,
        height:400
    },
    get: function(name) {
        var env = TXConfig.environment, obj = TXConfig[env] || TXConfig;
        return obj[name];
    }
};

/**
 * @type {Tx.Application}
 */
var TXApplication = null;

/**
 * @class
 */
Tx.Application = Tx.extend(Tx.Base, {
    /**
     * @lends Tx.Application.prototype
     * @param config
     */
    constructor: function(config){
        Tx.Base.prototype.constructor.call(this);
        Tx.apply(this, config);
    },
    initApplication: function() {
        this.remote = new Tx.Remote();
    },
    initNavigation: function() {

    },
    onLoad: function() {
        if (Tx.debugging){
            Tx.Debug.init();
            Tx.log('Debug.init');
        }
    },
    onUnload: function() {

    },
    onKeyDown: function(event) {

    },
    handleKeyDown: function(event){},
    handleKeyUp: function(event){},
    onReady: function() {
        Tx.log('onReady');
    },
    initEvents: function() {
        //document.onkeypress = this.handleKeyUp.bind(this);
        if (Tx.Device.isNetcast() && window.lgKb){
            lgKb.onKeyDown = this.handleKeyDown.bind(this);
        }
        else {
            if (Tx.Device.isPhilips()){
                document.onkeypress = this.handleKeyDown.bind(this);
                document.onkeyup = this.handleKeyUp.bind(this);
            }
            else {
                document.onkeydown = this.handleKeyDown.bind(this);
                document.onkeyup = this.handleKeyUp.bind(this);
                document.onkeypress = this.handleKeyUp.bind(this);
            }
        }
    },
    showError: function(html, noTimeout){
        var el = Tx.get('error');
        if (!el || !el.dom){
            el = document.createElement('div');
            Tx.getBody().appendChild(el);
            el = Tx.get(el);
            el.attr('id', 'error');
            var w = Math.floor(window.innerWidth / 2), h = Math.floor(window.innerHeight/2);
            el.setStyles({
                left:(w-250)+'px',
                top: (h-150)+'px'
            });
        }
        el.setHtml(html);
        el.show();
        var self = this;
        if (!noTimeout){
            setTimeout(self.hideError, 5000);
        }
    },
    hideError: function() {
        var el = Tx.get('error');
        if (el && el.dom){
            el.hide();
        }
    },
    showNetworkError: function() {
        this.showError('Netzwerkverbindung unterbrochen!', true);
    }
});/**
 * @author
 * @namespace
 */

Tx.Debug = {
    maxLines: 8,
    debugLines: [],
    debugServer: '/debugger/debugger_server.php',
    name: 'default',
    debugLogInfo:true,
    enabled: false,
    keyRedCount: 0,
    keyTimer:null,
    popupVisible: false,

    styles: {
        bottom:0,
        left:0,
        height: '50px',
        width:window.innerWidth+'px',
        position:'absolute',
        opacity:0.7,
        zIndex: 10000,
        overflow: 'hidden',
        backgroundColor: '#000',
        display: 'none',
        color: '#fff'
    },
    visible: false,
    init: function(cfg){
        Tx.apply(this, cfg);
        var div = document.createElement('div');
        div.id = 'tx-debug-container';
        div.className = 'debug-container';
        Tx.fly(div).setStyles(this.styles);
        Tx.getBody().appendChild(div);
        this.container = div;
        this.visible = false;
        if (this.debugServer){
            this.startDebug();
        }
        document.addEventListener('keydown', this.onKeyDown.bind(this),true);
        this.remote = new Tx.Remote();
        this.onKeyTimerBound = this.onKeyTimer.bind(this);
    },
    onKeyDown: function(event){
        var key = this.remote.getKeyFromEvent(event);
        switch(key){
            case TX_KEY_RED:
                if (this.keyTimer){
                    clearTimeout(this.keyTimer);
                }
                this.keyRedCount++;
                if (this.keyRedCount > 2){
                    if (this.popupVisible){
                        this.hidePopup();
                    }
                    else {
                        this.showPopup()
                    }
                    this.keyRedCount = 0;
                }
                else {
                    this.keyTimer = setTimeout(this.onKeyTimerBound, 1000);
                }
            break;
            case TX_KEY_UP: if (this.popup && this.popupVisible) {this.popup.prev(); Tx.stopEvent(event);} break;
            case TX_KEY_DOWN: if (this.popup && this.popupVisible) {this.popup.next(); Tx.stopEvent(event);} break;
            case TX_KEY_LEFT: this.hidePopup(); break;
            case TX_KEY_ENTER: if (this.popup && this.popupVisible) {this.popup.action(); Tx.stopEvent(event); this.hidePopup();} break;
        }
    },
    showPopup: function() {
        if (!this.popup){
            this.createPopup();
        }
        this.popup.show();
        this.popupVisible = true;
    },
    hidePopup: function() {
        this.popupVisible = false;
        if (this.popup){
            this.popup.hide();
        }
    },
    createPopup: function() {
        Tx.fly(Tx.getBody()).insertHtml('<div id="debug-popup-container" style="display:none; top:0; left:0; z-index:100; position:absolute; border: 1px solid #000;height:20px; width:200px;"></div>');
        this.popup = new (Tx.extend(Tx.Base, {
            container: Tx.get('debug-popup-container'),
            items: [
                'Reload',
                'Back',
                'Toggle Debug'
            ],
            current: 0,
            hide: function() {
                this.container.setStyle('display', 'none');
            },
            show: function() {
                this.container.setStyle('display', 'block');
            },
            setItem: function(){
                this.container.setHtml(this.items[this.current]);
            },
            prev: function() {
                this.current = Math.max(0, this.current-1);
                this.setItem()
            },
            next: function() {
                this.current = Math.min(this.items.length-1, this.current+1);
                this.setItem();
            },
            action: function() {
                switch(this.container.getHtml()){
                    case 'Reload': window.location.reload(); break;
                    case 'Back': window.history.back(); break;
                    case 'Toggle Debug': Tx.Debug.toggle(); break;
                }
            }
        }))();
        this.popup.setItem();
    },
    onKeyTimer: function() {
        clearTimeout(this.keyTimer);
        this.keyRedCount = 0;
    },
    hide: function() {
        this.visible = true;
        this.toggle();
    },
    toggle: function() {
        if (this.container) {
            this.visible = ! this.visible;
            this.container.style.display = this.visible ? 'block': 'none';
            if (!this.visible){
                this.container.innerHTML = '';
                this.log(window.navigator.userAgent);
            }
        }
    },
    /**
     *
     * @param data
     */
    log: function(data) {
        if (!this.enabled) return;
        if (this.debugLogInfo){
            this.debug.apply(this, arguments);
        }
        if (this.container){
            var line = this.getDebugLine(data);
            this.container.appendChild(line);
            if (this.container.scrollHeight > this.container.offsetHeight || this.container.children.length > 10){
                this.container.removeChild(this.container.childNodes[0]);
            }
        }
        if ('console' in window && console.log){
            window.console.log(data);
        }
        if (Tx.Device.isSamsung()){
            alert(data)
        }
    },
    debug: function() {
        if (!this.enabled || arguments.length == 0) return;
        if ('console' in window && console.debug){
            var type = arguments[0], data=[], i, l;
            for (i=1, l=arguments.length; i<l; i++){
                data.push(arguments[i]);
            }
            console[type].apply(console, data);
            //console.debug.apply(console,arguments);
        }
        this.debugLines.push(arguments);
        this.sendDebugLines();
    },
    startDebug: function() {
        var xhr = this.getXhr();
        xhr.request({
            url:this.debugServer,
            params:{
                command: 'start'
            },
            scope: this,
            success: function(request, data){
                this.ready = true;
            }
        });
    },
    sendDebugLines: function() {
        if (this.debugLines.length == 0){
            return;
        }
        var xhr = this.getXhr();
        if (xhr.running){
            return;
        }
        var args, i, l, j, len, params={};
        for (i=0, l=this.debugLines.length; i<l; i++){
            args = this.debugLines[i];
            for (j=0, len = args.length; j<len; j++) {
                params['data['+i+']['+j+']'] = this.getDebugValue(args[j]);
            }
        }
        this.debugLines = [];
        xhr.request({
            url:this.debugServer,
            params:params,
            debug:true,
            scope: this,
            success: function(request, data){
                this.sendDebugLines.delay(10, this);
            },
            failure: function(){
                this.sendDebugLines.delay(10, this);
            }
        });
    },
    getXhr: function(){
        if (!this.xhr  || Tx.Device.isSamsung()){
            this.xhr = new Tx.Ajax({debug: true});
        }
        return this.xhr;
    },
    getDebugValue: function(value) {
        switch(Tx.typeOf(value)){
            case 'element': return this.getDebugElement(value);
            case 'object': return this.getDebugObject(value);
            default: return this.encodeJson(value);
        }

    },
    getDebugElement: function(element){
        var result = [], v, value='';
        result.push('"_styleInfo":'+this.getDebugStyle(element));
        for (var k in element){
            if (k == 'innerHTML'  || k == 'textContent' || k== 'innerText' || k == 'outerHTML' || k == 'outerText' || k == 'eventListeners') continue;
            try {
                //alert(k);
                value = element[k];
                result.push(this.encodeJson(k,0)+':'+this.encodeJson(value,0));
            }
            catch(e){}
        }
        return '{'+result.join(',')+'}';
    },
    getDebugStyle: function(element){
        var el = Tx.fly(element), result = [], v;
        for (var k  in element.style){
            try {
                v= el.getCurrentStyle(k);
                if (v !== ''){
                    result.push(this.encodeJson(k,0)+':'+this.encodeJson(el.getCurrentStyle(k),0));
                }
            }
            catch(e){}
        }
        return '{'+result.join(',')+'}';
    },
    getDebugObject: function(obj){
        return this.objectEncodeJson(obj,1);
    },
    getDebugLine: function(data){
        var div = document.createElement('div'), type = typeof data;
        switch(type){
            case 'string': div.innerHTML = this.quoteStr(data); break;

        }
        return div;
    },
    quoteStr: function(str){
        return str.replace(/([<>&])/g,function(all, chr){
            switch(chr){
                case '<': return '&lt;';
                case '>': return '&gt;';
                case '&': return '&amp;';
                default: return '';
            }
        })
    },

    encodeJson: function(obj, level){
        level = level || 0;
        var specialChars = {'\b': '\\b', '\t': '\\t', '\n': '\\n', '\f': '\\f', '\r': '\\r', '"' : '\\"', '\\': '\\\\'};

        var replaceChars = function(chr){
            return specialChars[chr] || '\\u00' + Math.floor(chr.charCodeAt() / 16).toString(16) + (chr.charCodeAt() % 16).toString(16);
        };
        switch (Tx.typeOf(obj)){
            case 'string':
                return '"' + obj.replace(/[\x00-\x1f\\"]/g, replaceChars) + '"';
            case 'array':
                return level > 0  ? this.arrayEncodeJson(obj, level -1) : '"[Array]"';
            case 'object':
                return level > 0 ? this.objectEncodeJson(obj, level-1) : '"[Object]"';
            case 'number': case 'boolean': return String(obj);
            case 'null': return 'null';
            case 'element': return '"[HTMLElement '+obj.tagName+']"';
            case 'function': return '"[Function]"';
        }
        return null;
    },
    arrayEncodeJson: function(arr, level){
        var result = [], i, l;
        for (i=0, l=arr.length; i<l; i++){
            result.push(this.encodeJson(arr[i]), level);
        }
        return '['+result.join(',')+']'
    },
    objectEncodeJson: function(obj, level){
        var result = [], k, v;
        for (k in obj){
            try {
                v = this.encodeJson(obj[k],level);
            }
            catch (e){
                v = this.encodeJson(e.toString(),0);
            }
            if (v){
                result.push(this.encodeJson(k,0)+':'+v);
            }
        }
        return '{'+result.join(',')+'}';
    }

};

